@@ -10,9 +10,12 @@ from pymbolic.mapper.stringifier import (
 )
 from pymbolic.primitives import FloorDiv, Remainder
 
-from loki.ir import Stringifier, get_pragma_parameters
-from loki.tools import as_tuple, JoinableStringList, flatten
+from loki.backend.pprint import Stringifier
+from loki.backend.style import FortranStyle
+
 from loki.expression import LokiStringifyMapper, StringLiteral
+from loki.ir import get_pragma_parameters
+from loki.tools import as_tuple, JoinableStringList, flatten
 from loki.types import DataType, BasicType, DerivedType, ProcedureType
 
 
@@ -107,9 +110,11 @@ class FortranCodegen(Stringifier):
     """
     # pylint: disable=unused-argument
 
-    def __init__(self, depth=0, indent='  ', linewidth=90, conservative=True):
-        super().__init__(depth=depth, indent=indent, linewidth=linewidth,
-                         line_cont=' &\n{}& '.format, symgen=FCodeMapper())
+    def __init__(self, style, depth=0, conservative=True):
+        super().__init__(
+            style=style, depth=depth, line_cont=' &\n{}& '.format,
+            symgen=FCodeMapper()
+        )
         self.conservative = conservative
 
     def apply_label(self, line, label):
@@ -158,9 +163,9 @@ class FortranCodegen(Stringifier):
           END MODULE
         """
         header = self.format_line('MODULE ', o.name)
-        footer = self.format_line('END MODULE ', o.name)
+        footer = self.format_line('END MODULE ', o.name if self.style.module_end_named else '')
 
-        self.depth += 1
+        self.depth += self.style.module_spec_indent
 
         docstring = self.visit(o.docstring, **kwargs)
 
@@ -186,29 +191,73 @@ class FortranCodegen(Stringifier):
                 spec += self.visit(decl_part, **kwargs) + '\n'
         else:
             spec = self.visit(o.spec, **kwargs)
+        self.depth -= self.style.module_spec_indent
 
         # Render the routines
+        self.depth += self.style.module_contains_indent
         contains = self.visit(o.contains, **kwargs)
+        self.depth -= self.style.module_contains_indent
 
-        self.depth -= 1
         return self.join_lines(header, docstring, spec, contains, footer)
 
     def visit_Subroutine(self, o, **kwargs):
         """
         Format as
-          <ftype> [<prefix>] <name> ([<args>]) [RESULT(<name>)] [BIND(c, name=<name>)]
+          [<prefix>] SUBROUTINE <name> ([<args>]) [BIND(c, name=<name>)]
             ...docstring...
             ...spec...
             ...body...
           [CONTAINS]
             [...member...]
-          END <ftype> <name>
+          END SUBROUTINE <name>
         """
-        ftype = 'FUNCTION' if o.is_function else 'SUBROUTINE'
         prefix = self.join_items(o.prefix, sep=' ')
         if o.prefix:
             prefix += ' '
         arguments = self.join_items(o.argnames)
+        if isinstance(o.bind, str):
+            bind_c = f' BIND(c, name="{o.bind}")'
+        elif isinstance(o.bind, StringLiteral):
+            bind_c = f' BIND(c, name={o.bind})'
+        else:
+            bind_c = ''
+        header = self.format_line(prefix, 'SUBROUTINE ', o.name, ' (', arguments, ')', bind_c)
+        footer = self.format_line('END SUBROUTINE ', o.name if self.style.procedure_end_named else '')
+
+        self.depth += self.style.procedure_spec_indent
+        docstring = self.visit(o.docstring, **kwargs)
+        spec = self.visit(o.spec, **kwargs)
+        self.depth -= self.style.procedure_spec_indent
+
+        self.depth += self.style.procedure_body_indent
+        body = self.visit(o.body, **kwargs)
+        self.depth -= self.style.procedure_body_indent
+
+        self.depth += self.style.procedure_contains_indent
+        contains = self.visit(o.contains, **kwargs)
+        self.depth -= self.style.procedure_contains_indent
+        if contains:
+            return self.join_lines(header, docstring, spec, body, contains, footer)
+
+        return self.join_lines(header, docstring, spec, body, footer)
+
+    def visit_Function(self, o, **kwargs):
+        """
+        Format as
+          [<prefix>] FUNCTION <name> ([<args>]) [RESULT(<name>)] [BIND(c, name=<name>)]
+            ...docstring...
+            ...spec...
+            ...body...
+          [CONTAINS]
+            [...member...]
+          END FUNCTION <name>
+        """
+        prefix = self.join_items(o.prefix, sep=' ')
+        if o.prefix:
+            prefix += ' '
+        if not o.result_name in o.variable_map:
+            prefix += f'{self.visit(o.return_type)} '
+        arguments = self.join_items(o.argnames)
         result = f' RESULT({o.result_name})' if o.result_name\
                 and o.result_name.lower() != o.name.lower() else ''
         if isinstance(o.bind, str):
@@ -217,17 +266,24 @@ class FortranCodegen(Stringifier):
             bind_c = f' BIND(c, name={o.bind})'
         else:
             bind_c = ''
-        header = self.format_line(prefix, ftype, ' ', o.name, ' (', arguments, ')', result, bind_c)
-        footer = self.format_line('END ', ftype, ' ', o.name)
+        header = self.format_line(prefix, 'FUNCTION ', o.name, ' (', arguments, ')', result, bind_c)
+        footer = self.format_line('END FUNCTION ', o.name if self.style.procedure_end_named else '')
 
-        self.depth += 1
+        self.depth += self.style.procedure_spec_indent
         docstring = self.visit(o.docstring, **kwargs)
         spec = self.visit(o.spec, **kwargs)
+        self.depth -= self.style.procedure_spec_indent
+
+        self.depth += self.style.procedure_body_indent
         body = self.visit(o.body, **kwargs)
+        self.depth -= self.style.procedure_body_indent
+
+        self.depth += self.style.procedure_contains_indent
         contains = self.visit(o.contains, **kwargs)
-        self.depth -= 1
+        self.depth -= self.style.procedure_contains_indent
         if contains:
             return self.join_lines(header, docstring, spec, body, contains, footer)
+
         return self.join_lines(header, docstring, spec, body, footer)
 
     # Handler for AST base nodes
@@ -297,8 +353,9 @@ class FortranCodegen(Stringifier):
                     items += [k]
 
             # Ensure '!$<keyword> &' line continuation in final string
-            return str(JoinableStringList(items, sep=' ', width=self.linewidth,
-                                          cont=line_cont, separable=True))
+            return str(JoinableStringList(
+                items, sep=' ', width=self.style.linewidth, cont=line_cont, separable=True
+            ))
         return o.source.string
 
     def visit_CommentBlock(self, o, **kwargs):
@@ -534,9 +591,9 @@ class FortranCodegen(Stringifier):
         else:
             header = self.format_line('INTERFACE')
             footer = self.format_line('END INTERFACE')
-        self.depth += 1
+        self.depth += self.style.indent_default
         body = self.visit(o.body, **kwargs)
-        self.depth -= 1
+        self.depth -= self.style.indent_default
         return self.join_lines(header, body, footer)
 
     def visit_Loop(self, o, **kwargs):
@@ -554,13 +611,14 @@ class FortranCodegen(Stringifier):
         header = self.format_line(header_name, 'DO ', label, control)
         if o.has_end_do:
             footer_name = f' {o.name}' if o.name else ''
-            footer = self.format_line('END DO', footer_name)
+            enddo = 'END DO' if self.style.loop_end_space else 'ENDDO'
+            footer = self.format_line(enddo, footer_name)
             footer = self.apply_label(footer, o.loop_label)
         else:
             footer = None
-        self.depth += 1
+        self.depth += self.style.loop_indent
         body = self.visit(o.body, **kwargs)
-        self.depth -= 1
+        self.depth -= self.style.loop_indent
         return self.join_lines(pragma, header, body, footer, pragma_post)
 
     def visit_WhileLoop(self, o, **kwargs):
@@ -580,13 +638,14 @@ class FortranCodegen(Stringifier):
         header = self.format_line(header_name, 'DO', label, control)
         if o.has_end_do:
             footer_name = f' {o.name}' if o.name else ''
-            footer = self.format_line('END DO', footer_name)
+            enddo = 'END DO' if self.style.loop_end_space else 'ENDDO'
+            footer = self.format_line(enddo, footer_name)
             footer = self.apply_label(footer, o.loop_label)
         else:
             footer = None
-        self.depth += 1
+        self.depth += self.style.loop_indent
         body = self.visit(o.body, **kwargs)
-        self.depth -= 1
+        self.depth -= self.style.loop_indent
         return self.join_lines(pragma, header, body, footer, pragma_post)
 
     def visit_Conditional(self, o, **kwargs):
@@ -622,17 +681,18 @@ class FortranCodegen(Stringifier):
             header = f'{name[1:]}: IF' if name else 'IF'
             header = self.format_line(header, ' (', self.visit(o.condition, **kwargs), ') THEN')
 
-        self.depth += 1
+        self.depth += self.style.conditional_indent
         body = self.visit(o.body, **kwargs)
         if o.has_elseif:
-            self.depth -= 1
+            self.depth -= self.style.conditional_indent
             else_body = [self.visit(o.else_body, is_elseif=True, name=name, **kwargs)]
         else:
             else_body = [self.visit(o.else_body, **kwargs)]
-            self.depth -= 1
+            self.depth -= self.style.conditional_indent
             if o.else_body:
                 else_body = [self.format_line('ELSE', name)] + else_body
-            else_body += [self.format_line('END IF', name)]
+            endif = 'END IF' if self.style.conditional_end_space else 'ENDIF'
+            else_body += [self.format_line(endif, name)]
 
         return self.join_lines(header, body, *else_body)
 
@@ -658,9 +718,38 @@ class FortranCodegen(Stringifier):
         if o.else_body:
             cases.append(self.format_line('CASE DEFAULT', name))
         footer = self.format_line('END SELECT', name)
-        self.depth += 1
+        self.depth += self.style.indent_default
         bodies = self.visit_all(*o.bodies, o.else_body, **kwargs)
-        self.depth -= 1
+        self.depth -= self.style.indent_default
+        branches = [item for branch in zip(cases, bodies) for item in branch]
+        return self.join_lines(header, *branches, footer)
+
+    def visit_TypeConditional(self, o, **kwargs):
+        """
+        Format as
+          [name:] SELECT TYPE (<expr>)
+          [CLASS IS (<value>) [name]]
+            [...body...]
+          [TYPE IS (<value>) [name]]
+            [...body...]
+          [CLASS DEFAULT [name]]
+            [...body...]
+          END SELECT [name]
+        """
+        header_name = f'{o.name}: ' if o.name else ''
+        header = self.format_line(header_name, 'SELECT TYPE (', self.visit(o.expr, **kwargs), ')')
+        cases = []
+        name = f' {o.name}' if o.name else ''
+        for value in o.values:
+            case = self.visit(value[0], **kwargs)
+            guard = 'CLASS' if value[1] else 'TYPE'
+            cases.append(self.format_line(guard, ' IS (', case, ')', name))
+        if o.else_body:
+            cases.append(self.format_line('CLASS DEFAULT', name))
+        footer = self.format_line('END SELECT', name)
+        self.depth += self.style.indent_default
+        bodies = self.visit_all(*o.bodies, o.else_body, **kwargs)
+        self.depth -= self.style.indent_default
         branches = [item for branch in zip(cases, bodies) for item in branch]
         return self.join_lines(header, *branches, footer)
 
@@ -705,9 +794,9 @@ class FortranCodegen(Stringifier):
             cases += [self.format_line('ELSEWHERE')]
         footer = self.format_line('END WHERE')
 
-        self.depth += 1
+        self.depth += self.style.indent_default
         bodies = self.visit_all(*o.bodies, o.default, **kwargs)
-        self.depth -= 1
+        self.depth -= self.style.indent_default
 
         branches = [item for branch in zip(cases, bodies) for item in branch]
         return self.join_lines(*branches, footer)
@@ -740,9 +829,9 @@ class FortranCodegen(Stringifier):
         # Generate a multi-line FORALL construct
         name = f" {o.name}" if o.name is not None else ""
         footer = self.format_line('END FORALL', name)
-        self.depth += 1
+        self.depth += self.style.indent_default
         body = self.visit(o.body, **kwargs)
-        self.depth -= 1
+        self.depth -= self.style.indent_default
         return self.join_lines(header, body, footer)
 
     def visit_Section(self, o, **kwargs):
@@ -761,7 +850,9 @@ class FortranCodegen(Stringifier):
         assocs = [f'{self.visit(a[1], **kwargs)}=>{self.visit(a[0], **kwargs)}' for a in o.associations]
         header = self.format_line('ASSOCIATE (', self.join_items(assocs), ')')
         footer = self.format_line('END ASSOCIATE')
+        self.depth += self.style.associate_indent
         body = self.visit(o.body, **kwargs)
+        self.depth -= self.style.associate_indent
         return self.join_lines(header, body, footer)
 
     def visit_CallStatement(self, o, **kwargs):
@@ -861,6 +952,8 @@ class FortranCodegen(Stringifier):
             attributes += ['PRIVATE']
         if o.public:
             attributes += ['PUBLIC']
+        if o.protected:
+            attributes += ['PROTECTED']
 
         # Binding attributes
         if o.pass_attr is True:
@@ -901,9 +994,9 @@ class FortranCodegen(Stringifier):
             attrs = ''
         header = self.format_line('TYPE', attrs, ' ', o.name)
         footer = self.format_line('END TYPE ', o.name)
-        self.depth += 1
+        self.depth += self.style.indent_default
         body = self.visit(o.body, **kwargs)
-        self.depth -= 1
+        self.depth -= self.style.indent_default
         return self.join_lines(header, body, footer)
 
     def visit_BasicType(self, o, **kwargs):
@@ -928,7 +1021,7 @@ class FortranCodegen(Stringifier):
         """
         header = self.format_line('ENUM, BIND(C)')
         footer = self.format_line('END ENUM')
-        self.depth += 1
+        self.depth += self.style.indent_default
         body = []
         for var in o.symbols:
             name = self.visit(var, **kwargs)
@@ -937,15 +1030,18 @@ class FortranCodegen(Stringifier):
             else:
                 initial = f' = {self.visit(var.type.initial, **kwargs)}'
             body += [self.format_line('ENUMERATOR :: ', name, initial)]
-        self.depth -= 1
+        self.depth -= self.style.indent_default
         return self.join_lines(header, *body, footer)
 
 
-def fgen(ir, depth=0, conservative=False, linewidth=132):
+def fgen(ir, style=None, depth=0, conservative=False):
     """
     Generate standardized Fortran code from one or many IR objects/trees.
     """
-    return FortranCodegen(depth=depth, linewidth=linewidth, conservative=conservative).visit(ir) or ''
+    style = style if style else FortranStyle()
+    return FortranCodegen(
+        style=style, depth=depth, conservative=conservative
+    ).visit(ir) or ''
 
 
 """
