@@ -190,6 +190,21 @@ def test_simplify_collect_coefficients(source, ref):
 @pytest.mark.parametrize('source, ref', [
     ('1 == 1', 'True'),
     ('2 == 1', 'False'),
+    ('1 > 1', 'False'),
+    ('1 > 0', 'True'),
+    ('1 >= 1', 'True'),
+    ('5 >= 1', 'True'),
+    ('-1 >= 1', 'False'),
+    ('1 != 1', 'False'),
+    ('3 != 5', 'True'),
+    ('1 < 1', 'False'),
+    ('1 < 5', 'True'),
+    ('-1 < 10', 'True'),
+    ('-1 <= -2', 'False'),
+    ('-2 <= -2', 'True'),
+    ('-3 <= -2', 'True'),
+    ('1 <= 1', 'True'),
+    ('4 <= 3', 'False'),
     ('1 + 1 == 2', '1 + 1 == 2'),  # Not true without integer arithmetic
     ('.true. .and. .true.', 'True'),
     ('.true. .and. .false.', 'False'),
