@@ -8,19 +8,44 @@
 """
 Implementation of :any:`Source` and adjacent utilities
 """
+
 from bisect import bisect_left
-from itertools import accumulate
+from enum import Enum, auto
+from itertools import accumulate, takewhile
 import re
 from codetiming import Timer
 
 try:
-    from fparser.common.readfortran import FortranStringReader
+    from fparser.common.readfortran import Comment as ReadComment, FortranStringReader
 except ImportError:
+    ReadComment = None
     FortranStringReader = None
 
 from loki.logging import debug, warning
 
-__all__ = ['Source', 'FortranReader', 'source_to_lines', 'join_source_list']
+__all__ = [
+    'Source', 'SourceStatus', 'FortranReader', 'source_to_lines',
+    'join_source_list'
+]
+
+class SourceStatus(Enum):
+
+    """
+    The node and its children are unchanged - :any:`Source` is valid!
+    """
+    VALID = auto()
+
+    """
+    Interior node properties (expressions) have changed -
+    :any:`Source` is invalid!
+    """
+    INVALID_NODE = auto()
+
+    """
+    Interior node properties have not changed, but child nodes have
+    been altered - :any:`Source` is invalid!
+    """
+    INVALID_CHILDREN = auto()
 
 
 class Source:
@@ -35,13 +60,16 @@ class Source:
         The original raw source string
     file : str (optional)
         The file name
+    status : :any:`SourceStatus`
+        Flag indicating if the associated node has been altered
     """
 
-    def __init__(self, lines, string=None, file=None):
+    def __init__(self, lines, string=None, file=None, status=None):
         assert lines and len(lines) == 2 and (lines[1] is None or lines[1] >= lines[0])
         self.lines = lines
         self.string = string
         self.file = file
+        self.status = status if status else SourceStatus.VALID
 
     def clone(self, **kwargs):
         """
@@ -53,6 +81,8 @@ class Source:
             kwargs['string'] = self.string
         if self.file is not None and 'file' not in kwargs:
             kwargs['file'] = self.file
+        if self.status is not None and 'status' not in kwargs:
+            kwargs['status'] = self.status
         return type(self)(**kwargs)
 
     def __repr__(self):
@@ -127,6 +157,12 @@ class Source:
             for idx, line in enumerate(self.string.splitlines())
         ]
 
+    def invalidate(self):
+        """
+        Return an invalid copy of this source with ``status=SourceStatus=INVALID_NODE``.
+        """
+        return self.clone(status=SourceStatus.INVALID_NODE)
+
 
 class FortranReader:
     """
@@ -173,8 +209,18 @@ class FortranReader:
         """
         if FortranStringReader is None:
             raise RuntimeError('FortranReader needs fparser2')
-        reader = FortranStringReader(raw_source)
-        self.sanitized_lines = tuple(item for item in reader)
+        # do not ignore comments during reading as this would also ignore pragmas ...
+        reader = FortranStringReader(raw_source, ignore_comments=False)
+        lines = tuple(l for l in reader)
+        # ...but remove all comments that do not look like pragmas and are not inline comments
+        def is_not_comment(line, prev=None):
+            prev_end = prev.span[1] if prev else 0
+            return not isinstance(line, ReadComment) or (
+                line.span[0] > prev_end and line.line[:2] == '!$'
+            )
+        self.sanitized_lines = tuple(takewhile(is_not_comment, lines[:1])) + tuple(
+            l for l, prev in zip(lines[1:], lines) if is_not_comment(l, prev)
+        )
         self.sanitized_spans = (0,) + tuple(accumulate(len(item.line)+1 for item in self.sanitized_lines))
         self.sanitized_string = '\n'.join(item.line for item in self.sanitized_lines)
 
