@@ -15,7 +15,7 @@ import platform
 from time import perf_counter
 import pytest
 
-from loki import Module, Subroutine, Sourcefile, RawSource, config
+from loki import Function, Module, Subroutine, Sourcefile, RawSource, config
 from loki.frontend import (
     available_frontends, OMNI, FP, REGEX, RegexParserClass
 )
@@ -1192,10 +1192,13 @@ END SUBROUTINE DOT_PROD_SP_2D
     assert {
         routine.name.lower() for routine in source.subroutines
     } == {'dot_product_ecv', 'dot_prod_sp_2d'}
+    assert isinstance(source['dot_product_ecv'], Function)
+    assert isinstance(source['dot_prod_sp_2d'], Subroutine)
 
     source.make_complete()
-    routine = source['dot_product_ecv']
-    assert 'dot_product_ecv' in routine.variables
+    function = source['dot_product_ecv']
+    assert function.return_type.dtype == BasicType.REAL
+    assert function.return_type.kind == 'JPRB'
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
@@ -1322,3 +1325,120 @@ end subroutine
     assert routine.variable_map['str'].type.length == 10
     assert routine.variable_map['var'].type.dtype == BasicType.REAL
     assert routine.variable_map['var'].type.kind == 'jprb'
+
+
+def test_regex_sanitize_fypp_line_annotations():
+    """
+    Test that fypp line number annotations are sanitized correctly.
+    """
+
+    fcode = """
+module some_templated_mod
+
+# 1 "/path-to-hypp-macro/macro.hypp" 1
+# 2 "/path-to-hypp-macro/macro.hypp"
+# 3 "/path-to-hypp-macro/macro.hypp"
+# 5 "/path-to-fypp-template/template.fypp" 2
+
+integer :: a0
+integer :: a1
+integer :: a2
+integer :: a3
+integer :: a4
+
+end module some_templated_mod
+"""
+
+    module = Module.from_source(fcode, frontend=REGEX)
+    decls = FindNodes(ir.VariableDeclaration).visit(module.spec)
+
+    assert len(decls) == 5
+
+def test_regex_pragma():
+    """
+    Make sure the regex frontend can parse pragmas.
+    """
+    fcode = """
+SUBROUTINE FOO(A)
+
+INTEGER, INTENT(IN) :: A
+
+! make sure this won't end up as VariableDeclaration
+! INTEGER :: B
+! make sure this won't end up as VariableDeclaration
+!$loki INTEGER :: C
+! this is just a comment
+!$loki this-is-a-pragma
+!$acc this is another openacc pragma
+
+!$omp multiline &
+!$omp & pragma to be tested
+
+END SUBROUTINE FOO
+    """.strip()
+    source = Sourcefile.from_source(fcode, frontend=REGEX)
+    routine = source['FOO']
+
+    pragmas = FindNodes(ir.Pragma).visit(routine.ir)
+    var_decls = FindNodes(ir.VariableDeclaration).visit(routine.ir)
+
+    assert len(pragmas) == 4
+    assert pragmas[0].keyword == 'loki'
+    assert pragmas[0].content == 'INTEGER :: C'
+    assert pragmas[1].keyword == 'loki'
+    assert pragmas[1].content == 'this-is-a-pragma'
+    assert pragmas[2].keyword == 'acc'
+    assert pragmas[2].content == 'this is another openacc pragma'
+    assert pragmas[3].keyword == 'omp'
+    assert pragmas[3].content == 'multiline & pragma to be tested'
+
+    assert len(var_decls) == 1
+    assert var_decls[0].symbols == ('A',)
+
+    # compare with fully parsed source
+    source.make_complete()
+    compl_pragmas = FindNodes(ir.Pragma).visit(routine.ir)
+    for compl_pragma, pragma in zip(compl_pragmas, pragmas):
+        assert compl_pragma.keyword == pragma.keyword
+        assert compl_pragma.content == pragma.content
+
+def test_regex_comments():
+    """
+    Make sure the REGEX frontend doesn't match any comments
+    """
+    fcode = """
+SUBROUTINE my_routine
+! use my_mod
+use other_mod, only: foo
+use third_mod ! use fourth_mod
+use fifth_mod! , only: bar
+implicit none
+
+! type my_type
+type other_type
+end type
+
+integer :: var !, val
+
+! $acc not an acc pragma
+!$ acc also not an acc pragma
+var = 1 !$acc definitely not a pragma
+!!$acc not a pragma either
+!$$acc no pragma
+
+var = 1 &
+    &+1!$acc again no pragma
+
+call some_routine(var)
+var = var ! + function(val)
+var = var + 1 ! call other_routine(val)
+!call third routine(val)
+END SUBROUTINE my_routine
+    """.strip()
+
+    source = Sourcefile.from_source(fcode, frontend=REGEX)
+    routine = source['my_routine']
+    assert len(routine.imports) == 3
+    assert [imprt.module for imprt in routine.imports] == ['other_mod', 'third_mod', 'fifth_mod']
+    assert len(calls := FindNodes(ir.CallStatement).visit(routine.ir)) == 1 and calls[0].name == 'some_routine'
+    assert not FindNodes(ir.Pragma).visit(routine.ir)
