@@ -8,15 +8,14 @@
 # pylint: disable=too-many-lines
 from pathlib import Path
 import pytest
-import numpy as np
 
-from loki import Sourcefile, Module, Subroutine, fgen, fexprgen
-from loki.build import jit_compile, jit_compile_lib, clean_test
+from loki import Sourcefile, Module, Subroutine, Function, fgen, fexprgen
+from loki.jit_build import jit_compile, jit_compile_lib, clean_test
 from loki.expression import symbols as sym
 from loki.frontend import available_frontends, OMNI, REGEX
 from loki.ir import (
     nodes as ir, FindNodes, FindVariables, FindTypedSymbols,
-    Transformer
+    FindInlineCalls, Transformer
 )
 from loki.types import (
     BasicType, DerivedType, ProcedureType, SymbolAttributes
@@ -34,13 +33,19 @@ def fixture_header_path(here):
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_routine_simple(tmp_path, frontend):
+def test_routine_simple(frontend):
     """
     A simple standard looking routine to test argument declarations.
     """
     fcode = """
 subroutine routine_simple (x, y, scalar, vector, matrix)
-  ! This is the docstring
+  ! This is the docstring ...
+
+  ! It spans multiple intersected lines ...
+  ! ... and is followed by a ...
+
+  !$loki routine fun
+
   integer, parameter :: jprb = selected_real_kind(13,300)
   integer, intent(in) :: x, y
   real(kind=jprb), intent(in) :: scalar
@@ -56,51 +61,158 @@ end subroutine routine_simple
 
     # Test the internals of the subroutine
     routine = Subroutine.from_source(fcode, frontend=frontend)
-    assert isinstance(routine.body, ir.Section)
-    assert isinstance(routine.spec, ir.Section)
-    assert len(routine.docstring) == 1
-    assert routine.docstring[0].text == '! This is the docstring'
-    assert routine.definitions == ()
 
     assert routine.arguments == ('x', 'y', 'scalar', 'vector(x)', 'matrix(x, y)')
+    assert routine.variables == ('jprb', 'x', 'y', 'scalar', 'vector(x)', 'matrix(x, y)', 'i')
 
-    # Generate code, compile and load
-    filepath = tmp_path/(f'routine_simple_{frontend}.f90')
-    function = jit_compile(routine, filepath=filepath, objname='routine_simple')
+    # Check the docstring
+    assert len(routine.docstring) == 1
+    assert isinstance(routine.docstring[0], ir.CommentBlock)
+    if frontend == OMNI:
+        assert len(routine.docstring[0].comments) == 3
+        assert routine.docstring[0].comments[0].text == '! This is the docstring ...'
+        assert routine.docstring[0].comments[1].text == '! It spans multiple intersected lines ...'
+        assert routine.docstring[0].comments[2].text == '! ... and is followed by a ...'
+    else:
+        assert len(routine.docstring[0].comments) == 5
+        assert routine.docstring[0].comments[0].text == '! This is the docstring ...'
+        assert routine.docstring[0].comments[2].text == '! It spans multiple intersected lines ...'
+        assert routine.docstring[0].comments[3].text == '! ... and is followed by a ...'
+    assert routine.definitions == ()
 
-    # Test the generated identity results
-    x, y = 2, 3
-    vector = np.zeros(x, order='F')
-    matrix = np.zeros((x, y), order='F')
-    function(x=x, y=y, scalar=5., vector=vector, matrix=matrix)
-    assert np.all(vector == 5.)
-    assert np.all(matrix[0, :] == 5.)
-    assert np.all(matrix[1, :] == 10.)
-    clean_test(filepath)
+    # Check the spec
+    assert isinstance(routine.body, ir.Section)
+    if frontend == OMNI:
+        assert len(routine.spec.body) == 9
+        assert isinstance(routine.spec.body[0], ir.Intrinsic)
+        assert isinstance(routine.spec.body[1], ir.Pragma)
+        assert all(isinstance(n, ir.VariableDeclaration) for n in routine.spec.body[2:])
+        assert routine.spec.body[2].symbols == ('jprb',)
+        assert routine.spec.body[3].symbols == ('x',)
+        assert routine.spec.body[4].symbols == ('y',)
+        assert routine.spec.body[5].symbols == ('scalar',)
+        assert routine.spec.body[6].symbols == ('vector(x)',)
+        assert routine.spec.body[7].symbols == ('matrix(x, y)',)
+        assert routine.spec.body[8].symbols == ('i',)
+    else:
+        assert len(routine.spec.body) == 7
+        assert isinstance(routine.spec.body[0], ir.Pragma)
+        assert isinstance(routine.spec.body[1], ir.Comment)
+        assert all(isinstance(n, ir.VariableDeclaration) for n in routine.spec.body[2:])
+        assert routine.spec.body[2].symbols == ('jprb',)
+        assert routine.spec.body[3].symbols == ('x', 'y')
+        assert routine.spec.body[4].symbols == ('scalar',)
+        assert routine.spec.body[5].symbols == ('vector(x)', 'matrix(x, y)')
+        assert routine.spec.body[6].symbols == ('i',)
+
+    # Check the routine body
+    assert isinstance(routine.spec, ir.Section)
+    loops = FindNodes(ir.Loop).visit(routine.body)
+    assert len(loops) == 1 and loops[0].variable == 'i'
+    assigns = FindNodes(ir.Assignment).visit(routine.body)
+    assert len(assigns) == 2
+    assert assigns[0] in loops[0].body and assigns[1] in loops[0].body
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_routine_arguments(tmp_path, frontend):
+def test_routine_prefix(frontend):
+    """ Test matching of prefix attributes for subroutines """
+    fcode = """
+pure elemental subroutine my_routine(x, y)
+  implicit none
+  integer(kind=8), intent(inout) :: x, y
+
+  x = x + y
+end subroutine my_routine
+"""
+    # Note that Fparser fails here if the legal 'recursive' prefix is used!
+    routine = Subroutine.from_source(fcode, frontend=frontend)
+
+    assert routine.name == 'my_routine'
+    assert len(routine.prefix) == 2
+    assert routine.prefix == ('PURE', 'ELEMENTAL')
+
+    # Check that routine was parsed completely
+    assert isinstance(routine.body.body[-1], ir.Assignment)
+    assert routine.body.body[-1].lhs == 'x'
+    assert routine.body.body[-1].rhs == 'x + y'
+
+
+@pytest.mark.parametrize('frontend', available_frontends(
+    xfail=[(OMNI, 'OMNI frontend interface does not provide interfaces')]
+))
+def test_routine_bind(frontend):
+    """ Test matching of 'bind" suffix for subroutines in interfaces """
+    fcode = """
+module my_module
+  implicit none
+
+  interface
+    subroutine my_routine(x, y) bind(C, name='my_routine_c')
+      use, intrinsic :: iso_c_binding
+      integer(kind=c_int), intent(inout) :: x, y
+    end subroutine my_routine
+  end interface
+
+contains
+
+  subroutine my_routine(x, y)
+    integer(kind=4), intent(inout) :: x, y
+
+    x = x + y
+  end subroutine my_routine
+end module my_module
+"""
+    # Note that Fparser fails here if the legal 'recursive' prefix is used!
+    module = Module.from_source(fcode, frontend=frontend)
+
+    routine = module['my_routine']
+
+    intf_routine = module.interface_map['my_routine'].body[0]
+    # The bind attribute is named, check the name
+    assert isinstance(intf_routine.bind, sym.StringLiteral)
+    assert intf_routine.bind == 'my_routine_c'
+    assert "BIND(c, name='my_routine_c')" in fgen(intf_routine)
+    # TODO: bind(C) is not honoured atm
+
+    # TODO: Interface definition and module routine alias, is that intended?
+    assert intf_routine == routine
+
+    # Check that module routine was parsed completely
+    assert isinstance(routine.body.body[-1], ir.Assignment)
+    assert routine.body.body[-1].lhs == 'x'
+    assert routine.body.body[-1].rhs == 'x + y'
+
+
+@pytest.mark.parametrize('frontend', available_frontends())
+def test_routine_arguments(frontend):
     """
     A set of test to test internalisation and handling of arguments.
     """
 
     fcode = """
-subroutine routine_arguments (x, y, vector, matrix)
-  ! Test internal argument handling
+subroutine routine_arguments &
+ ! Test multiline dummy arguments with comments
+ & (x, y, scalar, &
+ ! Of course, not one...
+ ! but two comment lines
+ & vector, matrix)
+  implicit none
   integer, parameter :: jprb = selected_real_kind(13,300)
-  integer, intent(in) :: x, y
-  real(kind=jprb), dimension(x), intent(inout) :: vector
+  ! The order below is intentioanlly inverted
   real(kind=jprb), intent(inout) :: matrix(x, y)
+  real(kind=jprb), intent(in)    :: scalar
+  real(kind=jprb), dimension(x)  :: local_vector
+  real(kind=jprb), dimension(x), intent(out) :: vector
+  integer, intent(in) :: x, y
 
   integer :: i, j
-  real(kind=jprb), dimension(x) :: local_vector
   real(kind=jprb) :: local_matrix(x, y)
 
   do i=1, x
      local_vector(i) = i * 10.
      do j=1, y
-        local_matrix(i, j) = local_vector(i) + j * 2.
+        local_matrix(i, j) = local_vector(i) + j * scalar
      end do
   end do
 
@@ -111,90 +223,55 @@ end subroutine routine_arguments
 """
 
     routine = Subroutine.from_source(fcode, frontend=frontend)
+
+    # The line-creaking comments are attributed to the docstring.
+    # This behaviour is not ideal, but it is the current status quo!
+    if frontend == OMNI:
+        assert not routine.docstring
+    else:
+        assert len(routine.docstring) == 1
+        assert len(routine.docstring[0].comments) == 3
+        assert routine.docstring[0].comments[0].text == '! Test multiline dummy arguments with comments'
+        assert routine.docstring[0].comments[1].text == '! Of course, not one...'
+        assert routine.docstring[0].comments[2].text == '! but two comment lines'
+
+    # Argument order is determined by the dummies in the signature
+    assert routine.arguments == ('x', 'y', 'scalar', 'vector(x)', 'matrix(x, y)')
+    assert all(isinstance(a, sym.Scalar) for a in routine.arguments[0:3])
+    assert all(a.type.intent == 'in' for a in routine.arguments[0:3])
+    assert all(isinstance(a, sym.Array) for a in routine.arguments[3:])
+    assert all(a.type.dtype == BasicType.INTEGER for a in routine.arguments[0:2])
+    assert all(a.type.dtype == BasicType.REAL for a in routine.arguments[2:5])
+    if frontend == OMNI:
+        assert all(isinstance(a.type.kind, sym.InlineCall) for a in routine.arguments[2:5])
+    else:
+        assert all(a.type.kind == 'jprb' for a in routine.arguments[2:5])
+    assert routine.arguments[3].shape == ('x',)
+    assert routine.arguments[4].shape == ('x', 'y')
+    assert routine.arguments[3].type.intent == 'out'
+    assert routine.arguments[4].type.intent == 'inout'
+
+    # Local variable order is determined by the order of the declarations
     assert routine.variables == (
-        'jprb', 'x', 'y', 'vector(x)', 'matrix(x, y)',
-        'i', 'j', 'local_vector(x)', 'local_matrix(x, y)'
+        'jprb', 'matrix(x, y)', 'scalar', 'local_vector(x)',
+        'vector(x)', 'x', 'y', 'i', 'j', 'local_matrix(x, y)'
     )
-    assert routine.arguments == ('x', 'y', 'vector(x)', 'matrix(x, y)')
-
-    # Generate code, compile and load
-    filepath = tmp_path/(f'routine_arguments_{frontend}.f90')
-    function = jit_compile(routine, filepath=filepath, objname='routine_arguments')
-
-    # Test results of the generated and compiled code
-    x, y = 2, 3
-    vector = np.zeros(x, order='F')
-    matrix = np.zeros((x, y), order='F')
-    function(x=x, y=y, vector=vector, matrix=matrix)
-    assert np.all(vector == [10., 20.])
-    assert np.all(matrix == [[12., 14., 16.],
-                             [22., 24., 26.]])
-    clean_test(filepath)
-
-
-@pytest.mark.parametrize('frontend', available_frontends())
-def test_routine_arguments_multiline(tmp_path, frontend):
-    """
-    Test argument declarations with comments interjectected between dummies.
-    """
-    fcode = """
-subroutine routine_arguments_multiline &
- ! Test multiline dummy arguments with comments
- & (x, y, scalar, &
- ! Of course, not one...
- ! but two comment lines
- & vector, matrix)
-  integer, parameter :: jprb = selected_real_kind(13,300)
-  integer, intent(in) :: x, y
-  real(kind=jprb), intent(in) :: scalar
-  real(kind=jprb), intent(inout) :: vector(x), matrix(x, y)
-  integer :: i
-
-  do i=1, x
-     vector(i) = vector(i) + scalar
-     matrix(i, :) = i * vector(i)
-  end do
-end subroutine routine_arguments_multiline
-"""
-
-    # Test the internals of the subroutine
-    routine = Subroutine.from_source(fcode, frontend=frontend)
-    assert routine.arguments == ('x', 'y', 'scalar', 'vector(x)', 'matrix(x, y)')
-
-    # Generate code, compile and load
-    filepath = tmp_path/(f'routine_arguments_multiline_{frontend}.f90')
-    function = jit_compile(routine, filepath=filepath, objname='routine_arguments_multiline')
-
-    # Test results of the generated and compiled code
-    x, y = 2, 3
-    vector = np.zeros(x, order='F')
-    matrix = np.zeros((x, y), order='F')
-    function(x=x, y=y, scalar=5., vector=vector, matrix=matrix)
-    assert np.all(vector == 5.)
-    assert np.all(matrix[0, :] == 5.)
-    assert np.all(matrix[1, :] == 10.)
-    clean_test(filepath)
-
-
-@pytest.mark.parametrize('frontend', available_frontends())
-def test_routine_arguments_order(frontend):
-    """
-    Test argument ordering honours singateu (dummy list) instead of
-    order of apearance in spec declarations.
-    """
-    fcode = """
-subroutine routine_arguments_order(x, y, scalar, vector, matrix)
-  integer, parameter :: jprb = selected_real_kind(13,300)
-  integer, intent(in) :: x
-  real(kind=jprb), intent(inout) :: matrix(x, y)
-  real(kind=jprb), intent(in) :: scalar
-  integer, intent(in) :: y
-  real(kind=jprb), intent(inout) :: vector(x)
-  integer :: i
-end subroutine routine_arguments_order
-"""
-    routine = Subroutine.from_source(fcode, frontend=frontend)
-    assert routine.arguments == ('x', 'y', 'scalar', 'vector(x)', 'matrix(x, y)')
+    assert routine.variables[0].type.parameter
+    assert isinstance(routine.variables[0].type.initial, sym.InlineCall)
+    assert routine.variables[0].type.initial.function == 'selected_real_kind'
+    assert routine.variables[1].type.dtype == BasicType.REAL
+    assert routine.variables[1].shape == ('x', 'y')
+    assert routine.variables[2].type.dtype == BasicType.REAL
+    assert routine.variables[3].type.dtype == BasicType.REAL
+    assert routine.variables[3].shape == ('x',)
+    assert routine.variables[4].type.dtype == BasicType.REAL
+    assert routine.variables[4].shape == ('x',)
+    assert routine.variables[5].type.dtype == BasicType.INTEGER
+    assert routine.variables[6].type.dtype == BasicType.INTEGER
+    assert routine.variables[7].type.dtype == BasicType.INTEGER
+    assert routine.variables[8].type.dtype == BasicType.INTEGER
+    assert routine.variables[9].type.dtype == BasicType.REAL
+    assert routine.variables[9].shape == ('x', 'y')
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
@@ -261,48 +338,6 @@ integer, intent(in) :: c
     assert 'b(x)' in routine_vars
 
 
-@pytest.mark.parametrize('frontend', available_frontends())
-def test_routine_variables_local(tmp_path, frontend):
-    """
-    Test local variables and types
-    """
-    fcode = """
-subroutine routine_variables_local (x, y, maximum)
-  ! Test local variables and types
-  integer, parameter :: jprb = selected_real_kind(13,300)
-  integer, intent(in) :: x, y
-  real(kind=jprb), intent(out) :: maximum
-
-  integer :: i, j
-  real(kind=jprb), dimension(x) :: vector
-  real(kind=jprb) :: matrix(x, y)
-
-  do i=1, x
-     vector(i) = i * 10.
-     do j=1, y
-        matrix(i, j) = vector(i) + j * 2.
-     end do
-  end do
-  maximum = matrix(x, y)
-end subroutine routine_variables_local
-"""
-
-    # Test the internals of the subroutine
-    routine = Subroutine.from_source(fcode, frontend=frontend)
-    assert routine.variables == (
-        'jprb', 'x', 'y', 'maximum', 'i', 'j', 'vector(x)', 'matrix(x, y)'
-    )
-
-    # Generate code, compile and load
-    filepath = tmp_path/(f'routine_variables_local_{frontend}.f90')
-    function = jit_compile(routine, filepath=filepath, objname='routine_variables_local')
-
-    # Test results of the generated and compiled code
-    maximum = function(x=3, y=4)
-    assert np.all(maximum == 38.)  # 10*x + 2*y
-    clean_test(filepath)
-
-
 @pytest.mark.parametrize('frontend', available_frontends())
 def test_routine_variable_caching(frontend):
     """
@@ -488,13 +523,12 @@ subroutine routine_dim_shapes(v1, v2, v3, v4, v5)
 
 end subroutine routine_dim_shapes
 """
-    # TODO: Need a named subroutine lookup
     routine = Subroutine.from_source(fcode, frontend=frontend)
     assert routine.arguments == ('v1', 'v2', 'v3(:)', 'v4(v1, v2)', 'v5(0:v1, v2 - 1)')
 
     # Make sure variable/argument shapes on the routine work
     shapes = [fexprgen(v.shape) for v in routine.arguments if isinstance(v, sym.Array)]
-    assert shapes == ['(v1,)', '(v1, v2)', '(0:v1, v2 - 1)']
+    assert shapes == ['(:,)', '(v1, v2)', '(0:v1, v2 - 1)']
 
     # Ensure that all spec variables (including dimension symbols) are scoped correctly
     spec_vars = [v for v in FindVariables(unique=False).visit(routine.spec) if v.name.lower() != 'selected_real_kind']
@@ -504,7 +538,7 @@ end subroutine routine_dim_shapes
     # Ensure shapes of body variables are ok
     b_shapes = [fexprgen(v.shape) for v in FindVariables(unique=False).visit(routine.body)
                 if isinstance(v, sym.Array)]
-    assert b_shapes == ['(v1,)', '(v1,)', '(v1, v2)', '(0:v1, v2 - 1)']
+    assert b_shapes == ['(:,)', '(:,)', '(v1, v2)', '(0:v1, v2 - 1)']
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
@@ -921,7 +955,7 @@ contains
     integer, intent(in) :: in1
     integer, intent(out) :: out1
 
-    out1 = 5 * in1 + localvar
+    out1 = 5 * in1 + localvar + member_function(1)
   end subroutine member_procedure
 
   ! Below is disabled because f90wrap (wrongly) exhibits that
@@ -951,6 +985,12 @@ end subroutine routine_member_procedures
     assert routine.symbol_attrs.lookup('in1') is not None
     assert routine.members[0].get_symbol_scope('in1') is routine.members[0]
 
+    # Check that inline function is correctly identified
+    inline_calls = list(FindInlineCalls().visit(routine.members[0].body))
+    assert len(inline_calls) == 1
+    assert inline_calls[0].function.name == 'member_function'
+    assert inline_calls[0].function.type.dtype.procedure == routine.members[1]
+
     assert routine.members[1].name == 'member_function'
     assert routine.members[1].symbol_attrs.lookup('in2') is not None
     assert routine.members[1].get_symbol_scope('in2') is routine.members[1]
@@ -963,8 +1003,8 @@ end subroutine routine_member_procedures
 
     # Test results of the generated and compiled code
     out1, out2 = function(1, 2)
-    assert out1 == 7
-    assert out2 == 23
+    assert out1 == 12
+    assert out2 == 38
     clean_test(filepath)
 
 
@@ -1260,11 +1300,24 @@ END INTERFACE
 """.strip()
 
 
-@pytest.mark.parametrize('frontend', available_frontends(xfail=[(OMNI, 'Parser fails without dummy module provided')]))
-def test_subroutine_rescope_symbols(frontend):
-    """
-    Test the rescoping of variables.
+@pytest.mark.parametrize('frontend', available_frontends())
+def test_subroutine_rescope_symbols(tmp_path, frontend):
+    """ Test the rescoping of variables. """
+
+    fcode_module = """
+module some_mod
+implicit none
+contains
+  subroutine ext1(a)
+    integer, intent(inout) :: a(:)
+  end subroutine ext1
+
+  subroutine ext2(a)
+    integer, intent(inout) :: a(:)
+  end subroutine ext2
+end module some_mod
     """
+
     fcode = """
 subroutine test_subroutine_rescope(a, b, n)
   use some_mod, only: ext1
@@ -1300,7 +1353,8 @@ contains
 end subroutine test_subroutine_rescope
     """.strip()
 
-    routine = Subroutine.from_source(fcode, frontend=frontend)
+    Module.from_source(fcode_module, frontend=frontend, xmods=[tmp_path])
+    routine = Subroutine.from_source(fcode, frontend=frontend, xmods=[tmp_path])
     ref_fgen = fgen(routine)
 
     # Create a copy of the nested subroutine with rescoping and
@@ -1320,8 +1374,12 @@ end subroutine test_subroutine_rescope
             assert var.scope is nested_routine
 
     # Make sure the KIND parameter symbol in the variable's type is also correctly rescoped
-    assert routine.members[0].variable_map['j'].type.kind.scope is routine.members[0]
-    assert nested_routine.variable_map['j'].type.kind.scope is nested_routine
+    if frontend == OMNI:  # OMNI resolves paramter kind symbols
+        assert routine.members[0].variable_map['j'].type.kind == 4
+        assert nested_routine.variable_map['j'].type.kind == 4
+    else:
+        assert routine.members[0].variable_map['j'].type.kind.scope is routine.members[0]
+        assert nested_routine.variable_map['j'].type.kind.scope is nested_routine
 
     # Create another copy of the nested subroutine without rescoping
     nested_spec = Transformer().visit(routine.members[0].spec)
@@ -1331,7 +1389,10 @@ end subroutine test_subroutine_rescope
 
     # Save the kind symbol for later
     other_kind_var = other_routine.variable_map['j'].type.kind
-    assert other_kind_var.scope is routine.members[0]
+    if frontend == OMNI:
+        assert other_kind_var == 4
+    else:
+        assert other_kind_var.scope is routine.members[0]
 
     # Explicitly throw away type information from original nested routine
     routine.members[0]._parent = None
@@ -1353,11 +1414,14 @@ end subroutine test_subroutine_rescope
     assert all(var.scope is None or var.type is None for var in other_routine.variables)
 
     # Make sure changes apply also to the KIND attribute
-    assert routine.members[0].variable_map['j'].type.kind.scope is routine.members[0]
+    if frontend == OMNI:
+        assert routine.members[0].variable_map['j'].type.kind == 4
+    else:
+        assert routine.members[0].variable_map['j'].type.kind.scope is routine.members[0]
 
-    # This points (weakly) to an entry in routine.members[0].symbols which may or may not
-    # have been garbage collected at this point
-    assert other_kind_var.scope is not other_routine
+        # This points (weakly) to an entry in routine.members[0].symbols which may or may not
+        # have been garbage collected at this point
+        assert other_kind_var.scope is not other_routine
 
     # fgen of the not rescoped routine should lack some type information and thus either fail or
     # produce a different output, depending on whether GC has already happened
@@ -1373,11 +1437,23 @@ end subroutine test_subroutine_rescope
         )
 
 
-@pytest.mark.parametrize('frontend', available_frontends(xfail=[(OMNI, 'Parser fails without dummy module provided')]))
-def test_subroutine_rescope_clone(frontend):
-    """
-    Test the rescoping of variables in clone.
+@pytest.mark.parametrize('frontend', available_frontends())
+def test_subroutine_rescope_clone(tmp_path, frontend):
+    """ Test the rescoping of variables in clone. """
+    fcode_module = """
+module some_mod
+implicit none
+contains
+  subroutine ext1(a)
+    integer, intent(inout) :: a(:)
+  end subroutine ext1
+
+  subroutine ext2(a)
+    integer, intent(inout) :: a(:)
+  end subroutine ext2
+end module some_mod
     """
+
     fcode = """
 subroutine test_subroutine_rescope_clone(a, b, n)
   use some_mod, only: ext1
@@ -1412,7 +1488,8 @@ contains
 end subroutine test_subroutine_rescope_clone
     """.strip()
 
-    routine = Subroutine.from_source(fcode, frontend=frontend)
+    Module.from_source(fcode_module, frontend=frontend, xmods=[tmp_path])
+    routine = Subroutine.from_source(fcode, frontend=frontend, xmods=[tmp_path])
     ref_fgen = fgen(routine)
 
     # Create a copy of the nested subroutine with rescoping and
@@ -1534,116 +1611,6 @@ end subroutine valid_fortran
     assert "Declarations must have intents" in str(error.value)
 
 
-@pytest.mark.parametrize('frontend', available_frontends())
-def test_subroutine_prefix(frontend):
-    """
-    Test various prefixes that can occur in function/subroutine definitions
-    """
-    fcode = """
-pure elemental real function f_elem(a)
-    real, intent(in) :: a
-    f_elem = a
-end function f_elem
-    """.strip()
-
-    routine = Subroutine.from_source(fcode, frontend=frontend)
-    assert 'PURE' in routine.prefix
-    assert 'ELEMENTAL' in routine.prefix
-    assert routine.is_function is True
-    assert routine.return_type.dtype is BasicType.REAL
-
-    assert routine.name in routine.symbol_map
-    decl = [d for d in FindNodes(ir.VariableDeclaration).visit(routine.spec) if routine.name in d.symbols]
-    assert len(decl) == 1
-    decl = decl[0]
-
-    assert routine.procedure_type.is_function is True
-    assert routine.procedure_type.return_type.dtype is BasicType.REAL
-    assert routine.procedure_type.procedure is routine
-
-    assert routine.procedure_symbol.type.dtype.is_function is True
-    assert routine.procedure_symbol.type.dtype.return_type.dtype is BasicType.REAL
-    assert routine.procedure_symbol.type.dtype.procedure is routine
-
-    code = fgen(routine)
-    assert 'PURE' in code
-    assert 'ELEMENTAL' in code
-    assert fgen(decl) in code
-
-
-@pytest.mark.parametrize('frontend', available_frontends())
-def test_subroutine_suffix(frontend, tmp_path):
-    """
-    Test that subroutine suffixes are supported and correctly reproduced
-    """
-    fcode = """
-module subroutine_suffix_mod
-    implicit none
-
-    interface
-        function check_value(value) bind(C, name='check_value')
-            use, intrinsic :: iso_c_binding
-            real(c_float), value :: value
-            integer(c_int) :: check_value
-        end function check_value
-    end interface
-
-    interface
-        function fix_value(value) result(fixed) bind(C, name='fix_value')
-            use, intrinsic :: iso_c_binding
-            real(c_float), value :: value
-            real(c_float) :: fixed
-        end function fix_value
-    end interface
-contains
-    function out_of_physical_bounds(field, istartcol, iendcol, do_fix) result(is_bad)
-        real, intent(inout) :: field(:)
-        integer, intent(in) :: istartcol, iendcol
-        logical, intent(in) :: do_fix
-        logical :: is_bad
-
-        integer :: jcol
-        logical :: bad_value
-
-        is_bad = .false.
-        do jcol=istartcol,iendcol
-            bad_value = check_value(field(jcol)) > 0
-            is_bad = is_bad .or. bad_value
-            if (do_fix .and. bad_value) field(jcol) = fix_value(field(jcol))
-        end do
-    end function out_of_physical_bounds
-end module subroutine_suffix_mod
-    """.strip()
-    module = Module.from_source(fcode, frontend=frontend, xmods=[tmp_path])
-
-    check_value = module.interface_map['check_value'].body[0]
-    assert check_value.is_function
-    assert check_value.result_name == 'check_value'
-    assert check_value.return_type.dtype is BasicType.INTEGER
-    assert check_value.return_type.kind == 'c_int'
-    if frontend != OMNI:
-        assert check_value.bind == 'check_value'
-        assert "bind(c, name='check_value')" in fgen(check_value).lower()
-
-    fix_value = module.interface_map['fix_value'].body[0]
-    assert fix_value.is_function
-    assert fix_value.result_name == 'fixed'
-    assert fix_value.return_type.dtype is BasicType.REAL
-    assert fix_value.return_type.kind == 'c_float'
-    if frontend == OMNI:
-        assert "result(fixed)" in fgen(fix_value).lower()
-    else:
-        assert fix_value.bind == 'fix_value'
-        assert "result(fixed) bind(c, name='fix_value')" in fgen(fix_value).lower()
-
-    routine = module['out_of_physical_bounds']
-    assert routine.is_function
-    assert routine.result_name == 'is_bad'
-    assert routine.bind is None
-    assert routine.return_type.dtype is BasicType.LOGICAL
-    assert "result(is_bad)" in fgen(routine).lower()
-
-
 @pytest.mark.parametrize('frontend', available_frontends())
 def test_subroutine_comparison(frontend):
     """
@@ -1849,42 +1816,6 @@ END SUBROUTINE CLOUDSC
     assert all(isinstance(arg, sym.Array) for arg in routine.arguments[4:])
 
 
-@pytest.mark.parametrize('frontend', available_frontends())
-def test_subroutine_lazy_prefix(frontend):
-    """
-    Test that prefixes for functions are correctly captured when the object is made
-    complete.
-
-    This test represents a case where the REGEX frontend fails to capture these attributes correctly.
-
-    The rationale for this test is that we don't currently need these attributes
-    in the incomplete REGEX-parsed IR and we accept that this information is incomplete initially.
-    tmp_path, we make sure this information is captured correctly after completing the full frontend
-    parse.
-    """
-    fcode = """
-pure elemental real function f_elem(a)
-    real, intent(in) :: a
-    f_elem = a
-end function f_elem
-    """.strip()
-
-    routine = Subroutine.from_source(fcode, frontend=REGEX)
-    assert routine._incomplete
-    assert routine.prefix == ('pure elemental real',)
-    assert routine.arguments == ()
-    assert routine.is_function is True
-    assert routine.return_type is None
-
-    routine.make_complete(frontend=frontend)
-    assert not routine._incomplete
-    assert 'PURE' in routine.prefix
-    assert 'ELEMENTAL' in routine.prefix
-    assert routine.arguments == ('a',)
-    assert routine.is_function is True
-    assert routine.return_type.dtype is BasicType.REAL
-
-
 @pytest.mark.parametrize('frontend', available_frontends())
 def test_subroutine_clone_contained(frontend):
     fcode = """
@@ -2106,17 +2037,31 @@ end module field_array_module
     assert assigns[0].lhs.parent.type.dtype.typedef == field_3rb_tdef
 
 
-@pytest.mark.parametrize('frontend', available_frontends(
-    xfail=[(OMNI, 'OMNI cannot handle external type defs without source')]
-))
-def test_subroutine_deep_clone(frontend):
+@pytest.mark.parametrize('frontend', available_frontends())
+def test_subroutine_deep_clone(frontend, tmp_path):
     """
     Test that deep-cloning a subroutine actually ensures clean scope separation.
     """
+    fcode_module = """
+module my_types
+  implicit none
+  integer, parameter :: jprb=4
+
+  type nothing
+    logical :: different
+  end type nothing
+
+  type that_thing
+    integer :: n
+    integer :: else
+    type(nothing) :: entirely
+  end type that_thing
+end module my_types
+"""
 
     fcode = """
 subroutine myroutine(something)
-  use parkind1, only : jpim, jprb
+  use my_types, only : jprb, that_thing
   implicit none
 
   type(that_thing), intent(inout) :: something
@@ -2133,7 +2078,8 @@ subroutine myroutine(something)
   end associate
 end subroutine myroutine
 """
-    routine = Subroutine.from_source(fcode, frontend=frontend)
+    Module.from_source(fcode_module, frontend=frontend, xmods=[tmp_path])
+    routine = Subroutine.from_source(fcode, frontend=frontend, xmods=[tmp_path])
 
     # Create a deep-copy of the routine
     new_routine = routine.clone()
@@ -2295,14 +2241,29 @@ end subroutine
     assert not_tt_invalid.type.dtype == BasicType.DEFERRED
 
 
-@pytest.mark.parametrize('frontend', available_frontends(
-    xfail=[(OMNI, 'Parsing fails with no header information available')]
-))
+@pytest.mark.parametrize('frontend', available_frontends())
 def test_resolve_typebound_var_missing_definition(frontend, tmp_path):
     """
     Test correct behaviour of :any:`Scope.resolve_typebound_var` utility
     in the absence of type information
     """
+    fcode_module = """
+module header_mod
+    implicit none
+    type some_type
+        integer :: ival
+    end type some_type
+
+    type other_type
+        type(some_type) :: other
+    end type other_type
+
+    type third_type
+        type(other_type) :: some
+    end type third_type
+end module header_mod
+"""
+
     fcode = """
 subroutine some_routine
     use header_mod, only: third_type
@@ -2311,6 +2272,7 @@ subroutine some_routine
 end subroutine
     """.strip()
 
+    Module.from_source(fcode_module, frontend=frontend, xmods=[tmp_path])
     source = Sourcefile.from_source(fcode, frontend=frontend, xmods=[tmp_path])
     routine = source['some_routine']
 
@@ -2345,7 +2307,7 @@ subroutine member_functions
     end function
 end subroutine member_functions
     """.strip()
-    routine = Subroutine.from_source(fcode, frontend=frontend)
+    routine = Function.from_source(fcode, frontend=frontend)
     add_to_a = routine['add_to_a']
     return_type = add_to_a.procedure_type.return_type
     assert return_type.dtype == BasicType.REAL
