@@ -164,8 +164,9 @@ class DataOffloadTransformation(Transformation):
                             present = f' vars({", ".join(offload_args)})'
                         else:
                             present = ''
-                        pragma = Pragma(keyword='loki', content=f'device-present{present}')
-                        pragma_post = Pragma(keyword='loki', content='end device-present')
+                        pragma = Pragma(keyword='loki', content=f'structured-data {present}')
+                        pragma_post = Pragma(keyword='loki', content='end structured-data')
+
                 else:
                     copyin = f'in({", ".join(inargs)})' if inargs else ''
                     copy = f'inout({", ".join(inoutargs)})' if inoutargs else ''
