@@ -20,10 +20,10 @@ from loki.tools import CaseInsensitiveDict, as_tuple, flatten
 from loki.types import BasicType, DerivedType
 from loki.scope import SymbolAttributes
 
-from loki.transformations.hoist_variables import HoistVariablesTransformation
+from loki.transformations.temporaries.hoist_variables import HoistVariablesTransformation
 from loki.transformations.sanitise import do_resolve_associates
 from loki.transformations.single_column.base import SCCBaseTransformation
-from loki.transformations.single_column.vector import SCCDevectorTransformation
+from loki.transformations.single_column.devector import RemoveLoopTransformer
 from loki.transformations.utilities import single_variable_declaration
 from loki.ir.pragma_utils import get_pragma_parameters
 from loki.transformations.utilities import get_integer_variable
@@ -626,7 +626,7 @@ class SccLowLevelDataOffload(Transformation):
         do_resolve_associates(routine)
         SCCBaseTransformation.resolve_masked_stmts(routine, loop_variable=v_index)
         SCCBaseTransformation.resolve_vector_dimension(routine, loop_variable=v_index, bounds=self.horizontal.bounds)
-        SCCDevectorTransformation.kernel_remove_vector_loops(routine, self.horizontal)
+        routine.body = RemoveLoopTransformer(dimension=self.horizontal).visit(routine.body)
 
         self.kernel_cuf(
             routine, self.horizontal, self.block_dim, self.transformation_type,
