@@ -8,15 +8,15 @@
 import pytest
 
 from loki import Subroutine, Sourcefile, Dimension, fgen
-from loki.batch import ProcedureItem
+from loki.batch import ProcedureItem, TransformationError
 from loki.expression import Scalar, Array, IntLiteral
-from loki.frontend import available_frontends, OMNI
+from loki.frontend import available_frontends, OMNI, HAVE_FP
 from loki.ir import (
     FindNodes, Assignment, CallStatement, Conditional, Loop,
     Pragma, PragmaRegion, pragmas_attached, is_loki_pragma,
-    pragma_regions_attached
+    pragma_regions_attached, FindInlineCalls
 )
-
+from loki.logging import WARNING
 from loki.transformations import (
     DataOffloadTransformation, SanitiseTransformation,
     InlineTransformation, get_loop_bounds, PragmaModelTransformation
@@ -32,8 +32,9 @@ from loki.transformations.single_column import (
 @pytest.fixture(scope='module', name='horizontal')
 def fixture_horizontal():
     return Dimension(
-        name='horizontal', size='nlon', index='jl',
-        bounds=('start', 'end'), aliases=('nproma',)
+        name='horizontal', size=['dims%klon', 'nlon'], index='jl',
+        aliases=('nproma',), lower=('START', 'dims%ist'),
+        upper=('end', 'dims%iend')
     )
 
 @pytest.fixture(scope='module', name='horizontal_bounds_aliases')
@@ -236,6 +237,7 @@ def test_scc_demote_transformation(frontend, horizontal):
     assert fgen(assigns[10]).lower() == 'q(jl, nz) = q(jl, nz)*c + b(3)'
 
 
+@pytest.mark.xfail(not HAVE_FP, reason="Identification of array reduction intrinsics requires fparser.")
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('acc_data', ['default', 'copyin', None])
 def test_scc_annotate_openacc(frontend, horizontal, blocking, acc_data):
@@ -264,6 +266,127 @@ def test_scc_annotate_openacc(frontend, horizontal, blocking, acc_data):
   END SUBROUTINE column_driver
 """
 
+    fcode_kernel = """
+  SUBROUTINE compute_column(start, end, nlon, nproma, nlev, nz, q, other_var, more_var)
+    INTEGER, INTENT(IN) :: start, end   ! Iteration indices
+    INTEGER, INTENT(IN) :: nlon, nz     ! Size of the horizontal and vertical
+    INTEGER, INTENT(IN) :: nproma, nlev ! Aliases of horizontal and vertical sizes
+    REAL, INTENT(INOUT) :: q(nlon,nz)
+    REAL, INTENT(IN) :: other_var(nlon), more_var(nlon)
+    REAL :: t(nlon,nz)
+    REAL :: a(nlon)
+    REAL :: d(nproma)
+    REAL :: tmp(nproma)
+    REAL :: e(nlev)
+    REAL :: b(nlon,psize)
+    INTEGER, PARAMETER :: psize = 3
+    INTEGER :: jl, jk
+    REAL :: c
+    REAL :: tmp_sum
+
+    c = 5.345
+    DO jk = 2, nz
+      DO jl = start, end
+        t(jl, jk) = c * jk
+        q(jl, jk) = q(jl, jk-1) + t(jl, jk) * c
+      END DO
+    END DO
+
+    DO jl = start, end
+      tmp(jl) = other_var(jl)
+    END DO
+
+    tmp_sum = sum(tmp(start:end))
+
+    ! The scaling is purposefully upper-cased
+    DO JL = START, END
+      a(jl) = Q(JL, 1)
+      b(jl, 1) = Q(JL, 2)
+      b(jl, 2) = Q(JL, 3)
+      b(jl, 3) = a(jl) * (b(jl, 1) + b(jl, 2))
+
+      Q(JL, NZ) = Q(JL, NZ) * C + tmp
+    END DO
+  END SUBROUTINE compute_column
+"""
+    kernel = Subroutine.from_source(fcode_kernel, frontend=frontend)
+    driver = Subroutine.from_source(fcode_driver, frontend=frontend)
+    driver.enrich(kernel)  # Attach kernel source to driver call
+
+    # Test OpenACC annotations on non-hoisted version
+    scc_transform = (SCCDevectorTransformation(horizontal=horizontal),)
+    scc_transform += (SCCDemoteTransformation(horizontal=horizontal),)
+    scc_transform += (SCCRevectorTransformation(horizontal=horizontal),)
+    scc_transform += (SCCAnnotateTransformation(block_dim=blocking),)
+    scc_transform += (PragmaModelTransformation(directive='openacc'),)
+    for transform in scc_transform:
+        transform.apply(driver, role='driver', targets=['compute_column'])
+        transform.apply(kernel, role='kernel')
+
+    # Ensure routine is anntoated at vector level
+    pragmas = FindNodes(Pragma).visit(kernel.ir)
+    assert len(pragmas) == 6
+    assert pragmas[0].keyword == 'acc'
+    assert pragmas[0].content == 'routine vector'
+    assert pragmas[1].keyword == 'acc'
+    assert pragmas[1].content == 'data present(q, other_var, more_var)'
+    assert pragmas[-1].keyword == 'acc'
+    assert pragmas[-1].content == 'end data'
+
+    # Ensure vector and seq loops are annotated, including privatized variable `b`
+    with pragmas_attached(kernel, Loop):
+        kernel_loops = FindNodes(Loop).visit(kernel.ir)
+        assert len(kernel_loops) == 3
+        assert kernel_loops[0].pragma[0].keyword == 'acc'
+        assert kernel_loops[0].pragma[0].content == 'loop vector private(b)'
+        assert kernel_loops[1].pragma[0].keyword == 'acc'
+        assert kernel_loops[1].pragma[0].content == 'loop seq'
+        assert kernel_loops[2].pragma[0].keyword == 'acc'
+        assert kernel_loops[2].pragma[0].content == 'loop vector private(b)'
+
+    # Ensure array reduction intrinsic is still in the correct place
+    assert 'nproma' in kernel.variable_map['tmp'].dimensions
+    assert 'tmp(start:end)' in list(FindInlineCalls().visit(kernel.body))[0].parameters
+
+    # Ensure a single outer parallel loop in driver
+    with pragmas_attached(driver, Loop):
+        driver_loops = FindNodes(Loop).visit(driver.body)
+        assert len(driver_loops) == 1
+        assert driver_loops[0].pragma[0].keyword.lower() == 'acc'
+        if acc_data:
+            assert driver_loops[0].pragma[0].content == 'parallel loop gang vector_length(nlon)'
+        else:
+            assert driver_loops[0].pragma[0].content in (
+                'parallel loop gang private(other_var, more_var) vector_length(nlon)',
+                'parallel loop gang private(more_var, other_var) vector_length(nlon)'
+            )
+
+
+@pytest.mark.parametrize('frontend', available_frontends())
+@pytest.mark.parametrize('directive', [False, 'openacc', 'omp-gpu'])
+def test_scc_annotate_directive(frontend, horizontal, blocking, directive):
+    """
+    Test the correct addition of OpenACC pragmas to SCC format code (no hoisting).
+    """
+
+    fcode_driver = """
+  SUBROUTINE column_driver(nlon, nproma, nlev, nz, q, nb)
+    INTEGER, INTENT(IN)   :: nlon, nz, nb  ! Size of the horizontal and vertical
+    INTEGER, INTENT(IN)   :: nproma, nlev  ! Aliases of horizontal and vertical sizes
+    REAL, INTENT(INOUT)   :: q(nlon,nz,nb)
+    REAL :: other_var(nlon), more_var(nlon)
+    INTEGER :: b, start, end
+
+    start = 1
+    end = nlon
+
+    do b=1, nb
+      call compute_column(start, end, nlon, nproma, nz, q(:,:,b), other_var, more_var)
+    end do
+
+  END SUBROUTINE column_driver
+"""
+
     fcode_kernel = """
   SUBROUTINE compute_column(start, end, nlon, nproma, nlev, nz, q, other_var, more_var)
     INTEGER, INTENT(IN) :: start, end   ! Iteration indices
@@ -306,44 +429,97 @@ def test_scc_annotate_openacc(frontend, horizontal, blocking, acc_data):
     # Test OpenACC annotations on non-hoisted version
     scc_transform = (SCCDevectorTransformation(horizontal=horizontal),)
     scc_transform += (SCCDemoteTransformation(horizontal=horizontal),)
-    scc_transform += (SCCRevectorTransformation(horizontal=horizontal),)
+    scc_transform += (SCCSeqRevectorTransformation(horizontal=horizontal),)
     scc_transform += (SCCAnnotateTransformation(block_dim=blocking),)
-    scc_transform += (PragmaModelTransformation(directive='openacc'),)
+    scc_transform += (PragmaModelTransformation(directive=directive),)
     for transform in scc_transform:
         transform.apply(driver, role='driver', targets=['compute_column'])
         transform.apply(kernel, role='kernel')
 
-    # Ensure routine is anntoated at vector level
-    pragmas = FindNodes(Pragma).visit(kernel.ir)
-    assert len(pragmas) == 5
-    assert pragmas[0].keyword == 'acc'
-    assert pragmas[0].content == 'routine vector'
-    assert pragmas[1].keyword == 'acc'
-    assert pragmas[1].content == 'data present(q, other_var, more_var)'
-    assert pragmas[-1].keyword == 'acc'
-    assert pragmas[-1].content == 'end data'
+    if directive is None:
+        # Ensure routine is anntoated at vector level
+        pragmas = FindNodes(Pragma).visit(kernel.ir)
+        for pragma in pragmas:
+            print(f"pragma: {pragma.keyword} - {pragma.content}")
+        assert len(pragmas) == 4
+        assert pragmas[0].keyword == 'loki'
+        assert pragmas[0].content == 'routine seq'
 
-    # Ensure vector and seq loops are annotated, including privatized variable `b`
-    with pragmas_attached(kernel, Loop):
-        kernel_loops = FindNodes(Loop).visit(kernel.ir)
-        assert len(kernel_loops) == 2
-        assert kernel_loops[0].pragma[0].keyword == 'acc'
-        assert kernel_loops[0].pragma[0].content == 'loop vector private(b)'
-        assert kernel_loops[1].pragma[0].keyword == 'acc'
-        assert kernel_loops[1].pragma[0].content == 'loop seq'
+        # Ensure vector and seq loops are annotated, including privatized variable `b`
+        with pragmas_attached(kernel, Loop):
+            kernel_loops = FindNodes(Loop).visit(kernel.ir)
+            assert len(kernel_loops) == 1
+            assert kernel_loops[0].pragma[0].keyword == 'loki'
+            assert kernel_loops[0].pragma[0].content == 'loop seq'
 
-    # Ensure a single outer parallel loop in driver
-    with pragmas_attached(driver, Loop):
-        driver_loops = FindNodes(Loop).visit(driver.body)
-        assert len(driver_loops) == 1
-        assert driver_loops[0].pragma[0].keyword.lower() == 'acc'
-        if acc_data:
-            assert driver_loops[0].pragma[0].content == 'parallel loop gang vector_length(nlon)'
-        else:
+        # Ensure a single outer parallel loop in driver
+        with pragmas_attached(driver, Loop):
+            driver_loops = FindNodes(Loop).visit(driver.body)
+            assert len(driver_loops) == 2
+            assert driver_loops[0].pragma[0].keyword.lower() == 'loki'
+            assert driver_loops[0].pragma[0].content in (
+                'loop gang private(other_var, more_var) vlength(nlon)',
+                'loop gang private(more_var, other_var) vlength(nlon)'
+            )
+            assert driver_loops[1].pragma[0].keyword.lower() == 'loki'
+            assert driver_loops[1].pragma[0].content == 'loop vector'
+    if directive == 'openacc':
+        # Ensure routine is anntoated at vector level
+        pragmas = FindNodes(Pragma).visit(kernel.ir)
+        assert len(pragmas) == 4
+        assert pragmas[0].keyword == 'acc'
+        assert pragmas[0].content == 'routine seq'
+        assert pragmas[1].keyword == 'acc'
+        assert pragmas[1].content == 'data present(q, other_var, more_var)'
+        assert pragmas[-1].keyword == 'acc'
+        assert pragmas[-1].content == 'end data'
+
+        # Ensure vector and seq loops are annotated, including privatized variable `b`
+        with pragmas_attached(kernel, Loop):
+            kernel_loops = FindNodes(Loop).visit(kernel.ir)
+            assert len(kernel_loops) == 1
+            assert kernel_loops[0].pragma[0].keyword == 'acc'
+            assert kernel_loops[0].pragma[0].content == 'loop seq'
+
+        # Ensure a single outer parallel loop in driver
+        with pragmas_attached(driver, Loop):
+            driver_loops = FindNodes(Loop).visit(driver.body)
+            assert len(driver_loops) == 2
+            assert driver_loops[0].pragma[0].keyword.lower() == 'acc'
             assert driver_loops[0].pragma[0].content in (
                 'parallel loop gang private(other_var, more_var) vector_length(nlon)',
                 'parallel loop gang private(more_var, other_var) vector_length(nlon)'
             )
+            assert driver_loops[1].pragma[0].keyword.lower() == 'acc'
+            assert driver_loops[1].pragma[0].content == 'loop vector'
+    if directive == 'omp-gpu':
+        # Ensure routine is anntoated at vector level
+        pragmas = FindNodes(Pragma).visit(kernel.ir)
+        assert len(pragmas) == 4
+        assert pragmas[0].keyword == 'omp'
+        assert pragmas[0].content == 'declare target'
+        assert pragmas[1].keyword == 'loki'
+        assert pragmas[1].content == 'device-present vars(q, other_var, more_var)'
+        assert pragmas[-1].keyword == 'loki'
+        assert pragmas[-1].content == 'end device-present'
+
+        # Ensure vector and seq loops are annotated, including privatized variable `b`
+        with pragmas_attached(kernel, Loop):
+            kernel_loops = FindNodes(Loop).visit(kernel.ir)
+            assert len(kernel_loops) == 1
+            assert kernel_loops[0].pragma[0].keyword == 'loki'
+            assert kernel_loops[0].pragma[0].content == 'loop seq'
+
+        # Ensure a single outer parallel loop in driver
+        with pragmas_attached(driver, Loop):
+            driver_loops = FindNodes(Loop).visit(driver.body)
+            assert len(driver_loops) == 2
+            assert driver_loops[0].pragma[0].keyword.lower() == 'omp'
+            assert driver_loops[0].pragma[0].content == (
+                'target teams distribute thread_limit(nlon)'
+            )
+            assert driver_loops[1].pragma[0].keyword.lower() == 'omp'
+            assert driver_loops[1].pragma[0].content == 'parallel do'
 
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('directive', [None, 'openacc', 'omp-gpu'])
@@ -857,72 +1033,133 @@ def test_scc_multicond(frontend, horizontal, blocking):
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_scc_multiple_acc_pragmas(frontend, horizontal, blocking):
+@pytest.mark.parametrize('dims_type', ['pointer', 'allocatable', 'static'])
+@pytest.mark.parametrize('data_offload', [True, False])
+def test_scc_multiple_acc_pragmas(frontend, horizontal, blocking, dims_type, tmp_path, caplog,
+                                  data_offload):
     """
     Test that both '!$acc data' and '!$acc parallel loop gang' pragmas are created at the
     driver layer.
     """
 
-    fcode = """
-    subroutine test(work, nlon, nb)
+    fcode = f"""
+    module test_mod
+
+    type dims_type
+      integer :: klon
+      integer :: ist
+      integer :: iend
+      integer :: kbl
+    end type
+
+    contains
+
+    subroutine test(work, nb, dims)
     implicit none
 
-      integer, intent(in) :: nb, nlon
+      integer, intent(in) :: nb
+      type(dims_type), intent(in) :: dims
       real, dimension(nlon, nb), intent(inout) :: work
+      {'type(dims_type), pointer :: local_dims' if dims_type == 'pointer' else ''}
+      {'type(dims_type), allocatable :: local_dims' if dims_type == 'allocatable' else ''}
+      {'type(dims_type) :: local_dims' if dims_type == 'static' else ''}
       integer :: b
 
+      !$acc data present(dims)
       !$loki data
       !$omp parallel do private(b) shared(work, nproma)
         do b=1, nb
-           call some_kernel(nlon, work(:,b))
+           local_dims%ist = 1
+           local_dims%iend = dims%klon
+           local_dims%kbl = b
+           call some_kernel(local_dims, local_dims%klon, work(:,b))
         enddo
       !$omp end parallel do
       !$loki end data
+      !$acc end data
 
     end subroutine test
 
-    subroutine some_kernel(nlon, work)
+    subroutine some_kernel(dims, nlon, work)
     implicit none
 
+      type(dims_type), intent(in) :: dims
       integer, intent(in) :: nlon
-      real, dimension(nlon), intent(inout) :: work
+      real, dimension(nlon), target, intent(inout) :: work
+      real, pointer :: tmp(:) => null()
       integer :: jl
 
-      do jl=1,nlon
+      do jl=dims%ist,dims%iend
          work(jl) = work(jl) + 1.
       enddo
 
+      tmp => work
+
+      do jl=dims%ist,dims%iend
+         tmp(jl) = tmp(jl) + 1.
+      enddo
+
     end subroutine some_kernel
+    end module test_mod
     """
 
-    source = Sourcefile.from_source(fcode, frontend=frontend)
+    source = Sourcefile.from_source(fcode, frontend=frontend, xmods=[tmp_path])
     routine = source['test']
     routine.enrich(source.all_subroutines)
 
-    data_offload = DataOffloadTransformation(remove_openmp=True)
-    data_offload.transform_subroutine(routine, role='driver', targets=['some_kernel',])
+    if data_offload:
+        data_offload = DataOffloadTransformation(remove_openmp=True)
+        data_offload.transform_subroutine(routine, role='driver', targets=['some_kernel',])
     pragma_model = PragmaModelTransformation(directive='openacc')
     pragma_model.transform_subroutine(routine, role='driver', targets=['some_kernel',])
 
     scc_pipeline = SCCVectorPipeline(
-        horizontal=horizontal, block_dim=blocking, directive='openacc'
+        horizontal=horizontal, block_dim=blocking, directive='openacc',
+        privatise_derived_types=True
     )
-    scc_pipeline.apply(routine, role='driver', targets=['some_kernel',])
 
-    # Check that both acc pragmas are created
+    if dims_type in ['pointer', 'allocatable']:
+        with caplog.at_level(WARNING):
+            scc_pipeline.apply(routine, role='driver', targets=['some_kernel',])
+        if frontend == OMNI:
+            assert len(caplog.records) == 2
+            message = caplog.records[1].message
+        else:
+            assert len(caplog.records) == 1
+            message = caplog.records[0].message
+        assert "[Loki-SCC::Annotate] dynamically allocated structs are being privatised: ['local_dims']" in message
+    else:
+        scc_pipeline.apply(routine, role='driver', targets=['some_kernel',])
+
     pragmas = FindNodes(Pragma).visit(routine.ir)
-    assert len(pragmas) == 4
-    assert pragmas[0].keyword == 'acc'
-    assert pragmas[1].keyword == 'acc'
-    assert pragmas[2].keyword == 'acc'
-    assert pragmas[3].keyword == 'acc'
+    assert len(pragmas) == 6
 
-    assert 'data' in pragmas[0].content
-    assert 'copy' in pragmas[0].content
-    assert '(work)' in pragmas[0].content
-    assert pragmas[1].content == 'parallel loop gang vector_length(nlon)'
-    assert pragmas[2].content == 'end parallel loop'
-    assert pragmas[3].content == 'end data'
+    assert pragmas[0].keyword.lower() == 'acc'
+    assert pragmas[0].content == 'data present(dims)'
+    assert pragmas[5].content == 'end data'
+    assert pragmas[5].keyword.lower() == 'acc'
+
+    if data_offload:
+        assert all(p.keyword.lower() == 'acc' for p in pragmas[1:5])
+        assert pragmas[2].content == 'parallel loop gang private(local_dims) vector_length(dims%klon)'
+        assert pragmas[3].content == 'end parallel loop'
+        assert pragmas[4].content == 'end data'
+
+        assert 'data copy(work)' in pragmas[1].content
+    else:
+        assert pragmas[1].keyword == 'loki'
+        assert pragmas[2].keyword.lower() == 'omp'
+        assert pragmas[3].keyword.lower() == 'omp'
+        assert pragmas[4].keyword == 'loki'
+        assert pragmas[2].content == 'parallel do private(b) shared(work, nproma)'
+        assert pragmas[3].content == 'end parallel do'
+
+    # check that pointer association was correctly identified as a separator node
+    routine = source['some_kernel']
+    scc_pipeline.apply(routine, role='kernel')
+
+    loops = FindNodes(Loop).visit(routine.body)
+    assert len(loops) == 2
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
@@ -1014,11 +1251,12 @@ def test_scc_vector_reduction(frontend, pipeline, horizontal, blocking):
     """
 
     fcode = """
-    subroutine some_kernel(start, end, nlon, mij)
+    subroutine some_kernel(start, end, nlon, mij, lcond)
        integer, intent(in) :: nlon, start, end
        integer, dimension(nlon), intent(in) :: mij
+       logical, intent(in) :: lcond
 
-       integer :: jl, maxij
+       integer :: jl, maxij, sumij, sum0
 
        maxij = -1
        !$loki vector-reduction( mAx:maXij )
@@ -1027,6 +1265,23 @@ def test_scc_vector_reduction(frontend, pipeline, horizontal, blocking):
        enddo
        !$loki end vector-reduction( mAx:maXij )
 
+       do jl=start,end
+          mij(jl) = jl
+       enddo
+
+       if (lcond) then
+          sumij = 0
+          !$loki vector-reduction( +: sUmij, sUm0 )
+          do jl=start,end
+             sumij = sumij + mij(jl)
+          enddo
+          !$loki end vector-reduction( +: sUmij, sUm0 )
+       endif
+
+       do jl=start,end
+          mij(jl) = 0
+       enddo
+
     end subroutine some_kernel
     """
 
@@ -1038,31 +1293,34 @@ def test_scc_vector_reduction(frontend, pipeline, horizontal, blocking):
     routine = source['some_kernel']
 
     with pragma_regions_attached(routine):
-        region = FindNodes(PragmaRegion).visit(routine.body)
-        assert is_loki_pragma(region[0].pragma, starts_with = 'vector-reduction')
+        regions = FindNodes(PragmaRegion).visit(routine.body)
+        for region in regions:
+            assert is_loki_pragma(region.pragma, starts_with = 'vector-reduction')
 
 
-    scc_pipeline.apply(routine, role='kernel', targets=['some_kernel',])
+    if pipeline == SCCSVectorPipeline:
+        with pytest.raises(TransformationError):
+            scc_pipeline.apply(routine, role='kernel', targets=['some_kernel',])
+    else:
+        scc_pipeline.apply(routine, role='kernel', targets=['some_kernel',])
 
     pragmas = FindNodes(Pragma).visit(routine.body)
     if pipeline == SCCVVectorPipeline:
-        assert len(pragmas) == 3
+        assert len(pragmas) == 6
         assert all(p.keyword == 'acc' for p in pragmas)
 
         # Check OpenACC directives have been inserted
         with pragmas_attached(routine, Loop):
             loops = FindNodes(Loop).visit(routine.body)
-            assert len(loops) == 1
+            assert len(loops) == 4
             assert loops[0].pragma[0].content == 'loop vector reduction( mAx:maXij )'
+            assert loops[2].pragma[0].content == 'loop vector reduction( +: sUmij, sUm0 )'
 
-    if pipeline == SCCSVectorPipeline:
-        assert len(pragmas) == 4
-        assert pragmas[0].keyword == 'acc'
-        assert pragmas[1].keyword == 'loki'
-        assert 'vector-reduction' in pragmas[1].content
-        assert pragmas[2].keyword == 'loki'
-        assert 'vector-reduction' in pragmas[2].content
-        assert pragmas[3].keyword == 'acc'
+            conds = FindNodes(Conditional).visit(routine.body)
+            assert len(conds) == 1
+            loops = FindNodes(Loop).visit(conds[0].body)
+            assert len(loops) == 1
+            assert loops[0].pragma[0].content == 'loop vector reduction( +: sUmij, sUm0 )'
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
@@ -1166,9 +1424,9 @@ end subroutine kernel
     alias = Sourcefile.from_source(fcode_alias, frontend=frontend, xmods=[tmp_path]).subroutines[0]
 
     transform = SCCBaseTransformation(horizontal=horizontal)
-    with pytest.raises(RuntimeError):
+    with pytest.raises(TransformationError):
         transform.apply(no_start, role='kernel')
-    with pytest.raises(RuntimeError):
+    with pytest.raises(TransformationError):
         transform.apply(no_end, role='kernel')
 
     transform = SCCBaseTransformation(horizontal=horizontal_bounds_aliases)
@@ -1241,11 +1499,14 @@ def test_scc_inline_and_sequence_association(
 
     #Should fail because it can't resolve sequence association
     elif (inline_internals and not resolve_sequence_association):
-        with pytest.raises(RuntimeError) as e_info:
+        with pytest.raises(TransformationError) as e_info:
             inline_transform.apply(routine, role='kernel')
             scc_transform.apply(routine, role='kernel')
-        assert(e_info.exconly() ==
-               'RuntimeError: [Loki::TransformInline] Cannot resolve procedure call to contained_kernel')
+        assert(
+            (
+                '[Loki::TransformInline] Cannot resolve procedure call to contained_kernel'
+            ) in e_info.exconly()
+        )
 
     #Check that the call is properly modified
     elif (not inline_internals and resolve_sequence_association):
@@ -1269,3 +1530,39 @@ def test_scc_inline_and_sequence_association(
 
         assign = FindNodes(Assignment).visit(loop.body)[0]
         assert fgen(assign).lower() == 'work(jl) = 1.'
+
+
+@pytest.mark.xfail(not HAVE_FP, reason="Identification of array reduction intrinsics requires fparser.")
+@pytest.mark.parametrize('frontend', available_frontends())
+def test_scc_resolve_vector_dimension(frontend, horizontal):
+    """
+    Test SCC vector resolution utility.
+    """
+
+    fcode = """
+subroutine kernel(start, end, nlon, work)
+   integer, intent(in) :: start, end, nlon
+   real, intent(out) :: work(nlon)
+   integer :: jl
+   real :: work_maxval
+
+   work(start:end) = 0.
+   work_maxval = maxval(work(start:end))
+
+end subroutine kernel
+"""
+
+    routine = Subroutine.from_source(fcode, frontend=frontend)
+    SCCBaseTransformation.resolve_vector_dimension(routine, routine.variable_map[horizontal.index],
+                                                   horizontal.bounds)
+
+    loops = FindNodes(Loop).visit(routine.body)
+    assigns = FindNodes(Assignment).visit(routine.body)
+    assert len(assigns) == 2
+
+    assert len(loops) == 1
+    assert assigns[0] in loops[0].body
+    assert not assigns[1] in loops[0].body
+
+    assert 'maxval' == assigns[1].rhs.name.lower()
+    assert 'start:end' in assigns[1].rhs.parameters[0].dimensions
