@@ -14,7 +14,7 @@ from loki.batch import (
 from loki.frontend import available_frontends
 from loki.ir import (
     FindNodes, Assignment, CallStatement, Loop, Pragma,
-    pragmas_attached
+    pragmas_attached, Import
 )
 
 from loki.transformations import (
@@ -42,25 +42,40 @@ def fixture_vertical():
 
 @pytest.fixture(scope='module', name='blocking')
 def fixture_blocking():
-    return Dimension(name='blocking', size='nb', index='b')
+    return Dimension(name='blocking', size='nb', index='b', aliases=('block_var%nb',))
+
+
+@pytest.fixture(scope='module', name='blocking_alt')
+def fixture_blocking_alt():
+    return Dimension(name='blocking', size='dims%nb', index='b', aliases=('block_var%nb',))
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('hoist_pipeline', [SCCVHoistPipeline, SCCSHoistPipeline])
-def test_scc_hoist_multiple_kernels(frontend, horizontal, blocking, hoist_pipeline):
+def test_scc_hoist_multiple_kernels(frontend, horizontal, blocking_alt, hoist_pipeline, tmp_path):
     """
     Test hoisting of column temporaries to "driver" level.
     """
 
+    fcode_dims_mod = """
+  MODULE dims_mod
+    TYPE dims_type
+      integer :: nb
+    END TYPE
+  END MODULE dims_mod
+"""
+
     fcode_driver = """
-  SUBROUTINE column_driver(nlon, nz, q, nb)
-    INTEGER, INTENT(IN)   :: nlon, nz, nb  ! Size of the horizontal and vertical
-    REAL, INTENT(INOUT)   :: q(nlon,nz,nb)
+  SUBROUTINE column_driver(nlon, nz, q, dims)
+    USE dims_mod, ONLY : dims_type
+    INTEGER, INTENT(IN)   :: nlon, nz  ! Size of the horizontal and vertical
+    TYPE(DIMS_TYPE), INTENT(IN) :: dims
+    REAL, INTENT(INOUT)   :: q(nlon,nz,dims%nb)
     INTEGER :: b, start, end
 
     start = 1
     end = nlon
-    do b=1, nb
+    do b=1, dims%nb
       call compute_column(start, end, nlon, nz, q(:,:,b))
 
       ! A second call, to check multiple calls are honored
@@ -92,8 +107,11 @@ def test_scc_hoist_multiple_kernels(frontend, horizontal, blocking, hoist_pipeli
     END DO
   END SUBROUTINE compute_column
 """
+    dims_mod_source = Sourcefile.from_source(fcode_dims_mod, frontend=frontend, xmods=[tmp_path])
     kernel_source = Sourcefile.from_source(fcode_kernel, frontend=frontend)
-    driver_source = Sourcefile.from_source(fcode_driver, frontend=frontend)
+    dims_mod = dims_mod_source['dims_mod']
+    driver_source = Sourcefile.from_source(fcode_driver, frontend=frontend,
+                                           definitions=[dims_mod], xmods=[tmp_path])
     driver = driver_source['column_driver']
     kernel = kernel_source['compute_column']
     driver.enrich(kernel)  # Attach kernel source to driver call
@@ -102,7 +120,7 @@ def test_scc_hoist_multiple_kernels(frontend, horizontal, blocking, hoist_pipeli
     kernel_item = ProcedureItem(name='#compute_column', source=kernel_source)
 
     scc_hoist = hoist_pipeline(
-        horizontal=horizontal, block_dim=blocking, directive='openacc'
+        horizontal=horizontal, block_dim=blocking_alt, directive='openacc'
     )
 
     graph_dic = {driver_item: [kernel_item]}
@@ -151,7 +169,7 @@ def test_scc_hoist_multiple_kernels(frontend, horizontal, blocking, hoist_pipeli
     else:
         assert len(driver_loops) == 1
     assert driver_loops[0].variable == 'b'
-    assert driver_loops[0].bounds == '1:nb'
+    assert driver_loops[0].bounds == '1:dims%nb'
 
     # Ensure we have two kernel calls in the driver loop
     kernel_calls = FindNodes(CallStatement).visit(driver_loops[0])
@@ -165,7 +183,7 @@ def test_scc_hoist_multiple_kernels(frontend, horizontal, blocking, hoist_pipeli
     assert 't' in kernel.argnames
     assert kernel.variable_map['t'].type.intent.lower() == 'inout'
     assert kernel.variable_map['t'].type.shape == ('nlon', 'nz')
-    assert driver.variable_map['compute_column_t'].dimensions == ('nlon', 'nz', 'nb')
+    assert driver.variable_map['compute_column_t'].dimensions == ('nlon', 'nz', 'dims%nb')
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
@@ -176,11 +194,12 @@ def test_scc_hoist_multiple_kernels_loops(tmp_path, frontend, trim_vector_sectio
     """
 
     fcode_driver = """
-  SUBROUTINE driver(nlon, nz, q, nb)
+  SUBROUTINE driver(nlon, nz, q, nb, lflag)
     use kernel_mod, only: kernel
     implicit none
     INTEGER, INTENT(IN)   :: nlon, nz, nb  ! Size of the horizontal and vertical
     REAL, INTENT(INOUT)   :: q(nlon,nz,nb)
+    LOGICAL, INTENT(IN)   :: lflag
     REAL                  :: c, tmp(nlon,nz,nb)
     INTEGER :: b, jk, jl, start, end
 
@@ -219,13 +238,15 @@ def test_scc_hoist_multiple_kernels_loops(tmp_path, frontend, trim_vector_sectio
 
     !$loki driver-loop
     do b=3, nb
-      end = nlon - nb
-      !$loki separator
-      do jk = 2, nz
-        do jl = start, end
-          q(jl, jk, b) = 2.0 * jk * jl
+      if (lflag) then
+        end = nlon - nb
+        !$loki separator
+        do jk = 2, nz
+          do jl = start, end
+            q(jl, jk, b) = 2.0 * jk * jl
+          end do
         end do
-      end do
+      endif
     end do
   END SUBROUTINE driver
 """.strip()
@@ -372,7 +393,7 @@ END MODULE kernel_mod
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_scc_hoist_openacc(frontend, horizontal, vertical, blocking, tmp_path):
+def test_scc_hoist_openacc(frontend, horizontal, blocking, tmp_path):
     """
     Test the correct addition of OpenACC pragmas to SCC format code
     when hoisting array temporaries to driver.
@@ -380,16 +401,22 @@ def test_scc_hoist_openacc(frontend, horizontal, vertical, blocking, tmp_path):
 
     fcode_mod = """
 MODULE BLOCK_DIM_MOD
-    INTEGER :: nb
+    type block_type
+      INTEGER :: nb
+    end type block_type
 END MODULE BLOCK_DIM_MOD
     """.strip()
 
     fcode_driver = """
 SUBROUTINE column_driver(nlon, nz, q)
-    USE BLOCK_DIM_MOD, ONLY : nb
+    USE BLOCK_DIM_MOD, ONLY : block_type
     INTEGER, INTENT(IN)   :: nlon, nz  ! Size of the horizontal and vertical
-    REAL, INTENT(INOUT)   :: q(nlon,nz,nb)
+    REAL, INTENT(INOUT)   :: q(nlon,nz,block_var%nb)
     INTEGER :: b, start, end
+    type(block_type) :: block_var
+    INTEGER :: nb
+
+    nb = block_var%nb
 
     start = 1
     end = nlon
@@ -403,14 +430,17 @@ END SUBROUTINE column_driver
 SUBROUTINE compute_column(start, end, nlon, nz, q)
     INTEGER, INTENT(IN) :: start, end  ! Iteration indices
     INTEGER, INTENT(IN) :: nlon, nz    ! Size of the horizontal and vertical
-    REAL, INTENT(INOUT) :: q(nlon,nz)
+    REAL, TARGET, INTENT(INOUT) :: q(nlon,nz)
     REAL :: t(nlon,nz)
     REAL :: a(nlon)
     REAL :: b(nlon,psize)
+    REAL, POINTER :: b_ptr(:,:)
     INTEGER, PARAMETER :: psize = 3
     INTEGER :: jl, jk
     REAL :: c
 
+    b_ptr => q
+
     c = 5.345
     DO jk = 2, nz
       DO jl = start, end
@@ -456,7 +486,7 @@ end module my_scaling_value_mod
 
     scc_hoist = SCCHoistPipeline(
         horizontal=horizontal, block_dim=blocking,
-        directive='openacc', dim_vars=vertical.sizes
+        directive='openacc'
     )
 
     graph_dic = {driver_item: [kernel_item]}
@@ -468,9 +498,6 @@ end module my_scaling_value_mod
         sub_sgraph=graph
     )
 
-    # Check that blocking size has not been redefined
-    assert driver.symbol_map[blocking.size].type.module.name.lower() == 'block_dim_mod'
-
     with pragmas_attached(kernel, Loop):
         # Ensure kernel routine is anntoated at vector level
         kernel_pragmas = FindNodes(Pragma).visit(kernel.ir)
@@ -503,7 +530,7 @@ end module my_scaling_value_mod
         assert driver_pragmas[0].keyword == 'acc'
         assert driver_pragmas[0].content == 'enter data create(compute_column_t)'
         assert driver_pragmas[1].keyword == 'acc'
-        assert driver_pragmas[1].content == 'exit data delete(compute_column_t)'
+        assert driver_pragmas[1].content == 'exit data delete(compute_column_t) finalize'
 
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('as_kwarguments', [False, True])
@@ -551,11 +578,12 @@ def test_scc_hoist_nested_openacc(frontend, horizontal, vertical, blocking,
 
     fcode_inner_kernel = """
   SUBROUTINE update_q(start, end, nlon, nz, q, c)
+    use, intrinsic :: iso_fortran_env, only : real64
     INTEGER, INTENT(IN) :: start, end  ! Iteration indices
     INTEGER, INTENT(IN) :: nlon, nz    ! Size of the horizontal and vertical
     REAL, INTENT(INOUT) :: q(nlon,nz)
     REAL, INTENT(IN)    :: c
-    REAL :: t(nlon,nz)
+    REAL(kind=real64)   :: t(nlon,nz)
     INTEGER :: jl, jk
 
     DO jk = 2, nz
@@ -686,6 +714,13 @@ def test_scc_hoist_nested_openacc(frontend, horizontal, vertical, blocking,
         assert outer_kernel_pragmas[2].keyword == 'acc'
         assert outer_kernel_pragmas[2].content == 'end data'
 
+    # check that kind import was added to driver
+    imports = FindNodes(Import).visit(driver.spec)
+    assert imports
+    assert imports[0].module.lower() == 'iso_fortran_env'
+    assert 'real64' in imports[0].symbols
+    assert driver.variable_map['update_q_t'].type.kind.name.lower() == 'real64'
+
 
 @pytest.mark.parametrize('frontend', available_frontends())
 def test_scc_hoist_nested_inline_openacc(frontend, horizontal, vertical, blocking):
