@@ -110,6 +110,90 @@ end module rick_rolled
     (srcdir/'never_gonna_give.F90').unlink()
 
 
+@pytest.fixture(scope='module', name='source_with_args')
+def fixture_source_with_args(srcdir):
+    """
+    Write some source files with arguments to use in tests.
+    """
+
+    fcode_module = """
+module types_mod
+   type dims_type
+       integer :: kst
+       integer :: kend
+       integer :: klon
+   end type dims_type
+
+   type some_unused_type
+       real :: a
+   end type some_unused_type
+end module types_mod
+"""
+
+    fcode_driver = """
+subroutine driver(dims, StrUct)
+    use types_mod, only : dims_type, some_unused_type
+    implicit none
+    type(dims_type), intent(in) :: dims
+    type(some_unused_type), intent(in) :: struct
+    real, dimension(dims%klon) :: a, b, c, d
+
+
+    call kernel(dims%kst, dims%kend, dIms, sTRucT, a, b, c, d)
+
+end subroutine driver
+"""
+
+    fcode_kernel = """
+subroutine kernel(kst, kend, diMs, stRUCt, a, b, c, d)
+    use types_mod, only : dims_type, some_unused_type
+    implicit none
+    integer, intent(in) :: kst, kend
+    type(dims_type), intent(in) :: dIms
+    type(some_unused_type), intent(in) :: StrucT
+    real, intent(out), dimension(dims%klon) :: a, b, c, d
+    integer :: jrof
+
+    do jrof = kst, kend
+      a(jrof) = 0.
+      b(jrof) = 0.
+    enddo
+
+    !$loki remove
+    call an_unused_kernel(stRuCt)
+    !$loki end remove
+
+    call another_kernel(kst, kend, d=C, e=D)
+
+end subroutine kernel
+"""
+
+    fcode_another_kernel = """
+subroutine another_kernel(kst, kend, D, E)
+    implicit none
+    integer, intent(in) :: kst, kend
+    real, intent(out) :: d(:), e(:)
+    integer :: jrof
+
+    do jrof = kst, kend
+       d(jrof) = 0.
+    enddo
+end subroutine another_kernel
+"""
+
+    (srcdir/'module.F90').write_text(fcode_module)
+    (srcdir/'driver.F90').write_text(fcode_driver)
+    (srcdir/'kernel.F90').write_text(fcode_kernel)
+    (srcdir/'another_kernel.F90').write_text(fcode_another_kernel)
+
+    yield srcdir
+
+    (srcdir/'module.F90').unlink()
+    (srcdir/'driver.F90').unlink()
+    (srcdir/'kernel.F90').unlink()
+    (srcdir/'another_kernel.F90').unlink()
+
+
 @pytest.mark.parametrize('frontend', available_frontends())
 def test_transform_dead_code_conditional(frontend):
     """
@@ -227,6 +311,72 @@ end subroutine test_dead_code_conditional
     assert assigns[6].lhs == 'a' and assigns[6].rhs == 'a + 1.0'
 
 
+@pytest.mark.parametrize('frontend', available_frontends())
+def test_transform_dead_code_multiconditional(frontend):
+    """
+    Test correct elimination of unreachable conditional branches in
+    SELECT CASE statements.
+    """
+    fcode = """
+subroutine test_dead_code_multiconditional(a, b, i, flag)
+  real(kind=8), intent(inout) :: a, b
+  integer, intent(in) :: i
+  logical, intent(in) :: flag
+
+  if (flag) then
+    select case (2)
+    case (1)
+      a = a + b
+    case (5,2)
+      b = b + 2.0
+    case (3)
+      b = b + a
+    case default
+      a = a + 3.0
+    end select
+
+    select case (i)
+    case (1)
+      ! Check recursion...
+      if (2 == 2) then
+        b = b + a
+      else
+        a = a + 3.0
+      end if
+    case (2)
+      b = b + 4.0
+    case (3)
+      b = b + 5.0
+    case default
+      a = a + 6.0
+    end select
+
+  end if
+end subroutine test_dead_code_multiconditional
+"""
+    routine = Subroutine.from_source(fcode, frontend=frontend)
+    # Please note that nested conditionals (elseif) counts as two
+    assert len(FindNodes(ir.MultiConditional).visit(routine.body)) == 2
+    assert len(FindNodes(ir.Conditional).visit(routine.body)) == 2
+    assert len(FindNodes(ir.Assignment).visit(routine.body)) == 9
+
+    do_remove_dead_code(routine)
+
+    # Check that the first multi-conditional and the nested conditional
+    # inside the second conditional have been removed.
+    multiconds = FindNodes(ir.MultiConditional).visit(routine.body)
+    assert len(multiconds) == 1
+    assert multiconds[0].expr == 'i'
+    assert len(FindNodes(ir.Conditional).visit(routine.body)) == 1
+    assigns = FindNodes(ir.Assignment).visit(routine.body)
+    assert len(assigns) == 5
+    assert assigns[0].lhs == 'b' and assigns[0].rhs == 'b + 2.0'
+    assert assigns[1].lhs == 'b' and assigns[1].rhs == 'b + a'
+    assert assigns[2].lhs == 'b' and assigns[2].rhs == 'b + 4.0'
+    assert assigns[3].lhs == 'b' and assigns[3].rhs == 'b + 5.0'
+    assert assigns[4].lhs == 'a' and assigns[4].rhs == 'a + 6.0'
+
+
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('mark_with_comment', [True, False])
 def test_transform_remove_code_pragma_region(frontend, mark_with_comment):
@@ -439,3 +589,57 @@ def test_remove_code_transformation(frontend, source, include_intrinsics, kernel
     driver = scheduler['#rick_astley'].ir
     drhook_calls = [call for call in FindNodes(ir.CallStatement).visit(driver.body) if call.name == 'dr_hook']
     assert len(drhook_calls) == (2 if kernel_only else 0)
+
+
+@pytest.mark.parametrize('frontend', available_frontends())
+@pytest.mark.parametrize('kernel_override', [True, False])
+def test_remove_code_unused_args(frontend, source_with_args, kernel_override, tmp_path):
+    """
+    Test the removal of unused arguments in a call tree.
+    """
+
+    config = {
+        'default': {
+            'role': 'kernel', 'expand': True, 'strict': False,
+            'enable_imports': True, 'block': ['an_unused_kernel']
+        },
+        'routines': {
+            'driver': {'role': 'driver'},
+        }
+    }
+
+    if kernel_override:
+        config['routines'].update(
+            {'another_kernel': {'role': 'kernel', 'remove_unused_args': False}}
+        )
+
+    scheduler_config = SchedulerConfig.from_dict(config)
+    scheduler = Scheduler(paths=source_with_args, config=scheduler_config, frontend=frontend, xmods=[tmp_path])
+
+    # Apply the code removal transformation
+    transformation = RemoveCodeTransformation(remove_unused_args=True)
+    scheduler.process(transformation=transformation)
+
+    # check the kernel was transformed correctly
+    kernel = scheduler['#kernel'].ir
+    driver = scheduler['#driver'].ir
+
+    kernel_calls = FindNodes(ir.CallStatement).visit(kernel.body)
+    driver_calls = FindNodes(ir.CallStatement).visit(driver.body)
+
+    assert len(kernel_calls) == 1
+    assert kernel_calls[0].name.name.lower() == 'another_kernel'
+    assert len(driver_calls) == 1
+    assert driver_calls[0].name.name.lower() == 'kernel'
+
+    kernel_vars = [v.clone(dimensions=None) for v in kernel.variables]
+
+    if kernel_override:
+        assert not 'struct' in kernel_vars
+        assert not 'struct' in driver_calls[0].arguments
+
+        assert 'd' in kernel_vars
+        assert 'd' in driver_calls[0].arguments
+    else:
+        assert not any(v in kernel_vars for v in ['d', 'struct'])
+        assert not any(v in driver_calls[0].arguments for v in ['d', 'struct'])
