@@ -34,7 +34,7 @@ __all__ = [
     'sanitise_imports', 'replace_selected_kind',
     'single_variable_declaration', 'recursive_expression_map_update',
     'get_integer_variable', 'get_loop_bounds', 'find_driver_loops',
-    'get_local_arrays', 'check_routine_sequential'
+    'get_local_arrays', 'check_routine_sequential', 'substitute_variables_for_definitions'
 ]
 
 
@@ -550,7 +550,11 @@ def get_integer_variable(routine, name):
     name : string
         Name of the variable to find the in the routine.
     """
-    if not (v_index := routine.symbol_map.get(name, None)):
+
+    symbol_map = routine.symbol_map
+    if name.split('%', maxsplit=1)[0] in symbol_map:
+        v_index = routine.resolve_typebound_var(name, symbol_map)
+    else:
         dtype = SymbolAttributes(BasicType.INTEGER)
         v_index = sym.Variable(name=name, type=dtype, scope=routine)
     return v_index
@@ -680,3 +684,25 @@ def check_routine_sequential(routine):
             return True
 
     return False
+
+def substitute_variables_for_definitions(routine, variables):
+    """
+    Substitute variables for definitions if applicable.
+
+    Parameters
+    ----------
+    routine : :any:`Subroutine`
+        Subroutine to remap the variables.
+    variables : :any:`Expression`, list, tuple
+        List of variables to remap.
+    """
+    variables = as_tuple(variables)
+    var_map = {
+        assignment.lhs: assignment.rhs
+        for assignment in FindNodes(ir.Assignment).visit(routine.body)
+        if assignment.lhs in variables
+    }
+    if var_map:
+        remapped_variables = [var_map[var] if var in var_map else var for var in variables]
+        return remapped_variables
+    return variables
